# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from sahara.plugins import base as base_plugins
from sahara.service import ops
from sahara.tests.unit import base
from sahara.utils import cluster as c_u


class FakeCluster(object):
    id = 'id'
    status = "Some_status"
    name = "Fake_cluster"


class FakeNodeGroup(object):
    id = 'id'
    count = 2
    instances = [{'instance_name': 'id-10', 'id': 2},
                 {'instance_name': 'id-2', 'id': 1}]


class FakePlugin(mock.Mock):
    node_groups = [FakeNodeGroup()]
    is_transient = False

    def update_infra(self, cluster):
        TestOPS.SEQUENCE.append('update_infra')

    def configure_cluster(self, cluster):
        TestOPS.SEQUENCE.append('configure_cluster')

    def start_cluster(self, cluster):
        TestOPS.SEQUENCE.append('start_cluster')

    def on_terminate_cluster(self, cluster):
        TestOPS.SEQUENCE.append('on_terminate_cluster')

    def decommission_nodes(self, cluster, instances_to_delete):
        TestOPS.SEQUENCE.append('decommission_nodes')

    def scale_cluster(self, cluster, node_group_id_map,
                      node_group_instance_map=None):
        TestOPS.SEQUENCE.append('plugin.scale_cluster')

    def cluster_destroy(self, ctx, cluster):
        TestOPS.SEQUENCE.append('cluster_destroy')


class FakeINFRA(object):
    def create_cluster(self, cluster):
        TestOPS.SEQUENCE.append('create_cluster')

    def scale_cluster(self, cluster, node_group_id_map,
                      node_group_instance_map=None):
        TestOPS.SEQUENCE.append('INFRA.scale_cluster')
        return True

    def shutdown_cluster(self, cluster, force):
        TestOPS.SEQUENCE.append('shutdown_cluster')

    def rollback_cluster(self, cluster, reason):
        TestOPS.SEQUENCE.append('rollback_cluster')


class TestOPS(base.SaharaWithDbTestCase):
    SEQUENCE = []

    @mock.patch('sahara.service.ops._refresh_health_for_cluster')
    @mock.patch('sahara.utils.cluster.change_cluster_status_description',
                return_value=FakeCluster())
    @mock.patch('sahara.service.ops._update_sahara_info')
    @mock.patch('sahara.service.ops._prepare_provisioning',
                return_value=(mock.Mock(), mock.Mock(), FakePlugin()))
    @mock.patch('sahara.utils.cluster.change_cluster_status')
    @mock.patch('sahara.conductor.API.cluster_get')
    @mock.patch('sahara.service.ops.CONF')
    @mock.patch('sahara.service.trusts.create_trust_for_cluster')
    @mock.patch('sahara.conductor.API.job_execution_get_all')
    @mock.patch('sahara.service.edp.job_manager.run_job')
    def test_provision_cluster(self, p_run_job, p_job_exec, p_create_trust,
                               p_conf, p_cluster_get, p_change_status,
                               p_prep_provisioning, p_update_sahara_info,
                               p_change_cluster_status_desc, refresh):
        del self.SEQUENCE[:]
        ops.INFRA = FakeINFRA()
        ops._provision_cluster('123')
        # checking that order of calls is right
        self.assertEqual(['update_infra', 'create_cluster',
                          'configure_cluster', 'start_cluster'], self.SEQUENCE,
                         'Order of calls is wrong')
        self.assertEqual(1, refresh.call_count)

    @mock.patch('sahara.service.ops._refresh_health_for_cluster')
    @mock.patch('sahara.service.ntp_service.configure_ntp')
    @mock.patch('sahara.service.ops.CONF')
    @mock.patch('sahara.service.ops._prepare_provisioning',
                return_value=(mock.Mock(), mock.Mock(), FakePlugin()))
    @mock.patch('sahara.utils.cluster.change_cluster_status',
                return_value=FakePlugin())
    @mock.patch('sahara.utils.cluster.get_instances')
    def test_provision_scaled_cluster(self, p_get_instances, p_change_status,
                                      p_prep_provisioning, p_conf, p_ntp,
                                      refresh):
        del self.SEQUENCE[:]
        ops.INFRA = FakeINFRA()
        p_conf.use_identity_api_v3 = True
        ops._provision_scaled_cluster('123', {'id': 1})
        # checking that order of calls is right
        self.assertEqual(['decommission_nodes', 'INFRA.scale_cluster',
                          'plugin.scale_cluster'], self.SEQUENCE,
                         'Order of calls is wrong')
        self.assertEqual(1, refresh.call_count)

    @mock.patch('sahara.service.ops._setup_trust_for_cluster')
    @mock.patch('sahara.service.ops.CONF')
    @mock.patch('sahara.service.trusts.delete_trust_from_cluster')
    @mock.patch('sahara.context.ctx')
    def test_terminate_cluster(self, p_ctx, p_delete_trust, p_conf, p_set):
        del self.SEQUENCE[:]
        base_plugins.PLUGINS = FakePlugin()
        base_plugins.PLUGINS.get_plugin.return_value = FakePlugin()
        ops.INFRA = FakeINFRA()
        ops.conductor = FakePlugin()
        ops.terminate_cluster('123')
        # checking that order of calls is right
        self.assertEqual(['on_terminate_cluster', 'shutdown_cluster',
                         'cluster_destroy'], self.SEQUENCE,
                         'Order of calls is wrong')

    @mock.patch('sahara.utils.cluster.change_cluster_status_description')
    @mock.patch('sahara.service.ops._prepare_provisioning')
    @mock.patch('sahara.utils.cluster.change_cluster_status')
    @mock.patch('sahara.service.ops._rollback_cluster')
    @mock.patch('sahara.conductor.API.cluster_get')
    def test_ops_error_hadler_success_rollback(
            self, p_cluster_get, p_rollback_cluster, p_change_cluster_status,
            p__prepare_provisioning, p_change_cluster_status_desc):
        # Test scenario: failed scaling -> success_rollback
        fake_cluster = FakeCluster()
        p_change_cluster_status_desc.return_value = FakeCluster()
        p_rollback_cluster.return_value = True
        p_cluster_get.return_value = fake_cluster
        p__prepare_provisioning.side_effect = ValueError('error1')

        expected = [
            mock.call(fake_cluster, c_u.CLUSTER_STATUS_ACTIVE,
                      'Scaling cluster failed for the following '
                      'reason(s): error1')
        ]

        ops._provision_scaled_cluster(fake_cluster.id, {'id': 1})
        self.assertEqual(expected, p_change_cluster_status.call_args_list)

    @mock.patch('sahara.utils.cluster.change_cluster_status_description')
    @mock.patch('sahara.service.ops._prepare_provisioning')
    @mock.patch('sahara.utils.cluster.change_cluster_status')
    @mock.patch('sahara.service.ops._rollback_cluster')
    @mock.patch('sahara.conductor.API.cluster_get')
    def test_ops_error_hadler_failed_rollback(
            self, p_cluster_get, p_rollback_cluster, p_change_cluster_status,
            p__prepare_provisioning, p_change_cluster_status_desc):
        # Test scenario: failed scaling -> failed_rollback
        fake_cluster = FakeCluster()
        p_change_cluster_status_desc.return_value = FakeCluster()
        p__prepare_provisioning.side_effect = ValueError('error1')
        p_rollback_cluster.side_effect = ValueError('error2')
        p_cluster_get.return_value = fake_cluster

        expected = [
            mock.call(
                fake_cluster, 'Error', 'Scaling cluster failed for the '
                                       'following reason(s): error1, error2')
        ]

        ops._provision_scaled_cluster(fake_cluster.id, {'id': 1})
        self.assertEqual(expected, p_change_cluster_status.call_args_list)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from heatclient import exc as heat_exc
import mock

from sahara.service import engine
from sahara.service.heat import heat_engine
from sahara.tests.unit import base


class EngineTest(engine.Engine):
    def __init__(self):
        super(EngineTest, self).__init__()
        self.order = []

    def create_cluster(self, cluster):
        pass

    def get_type_and_version(self):
        pass

    def rollback_cluster(self, cluster, reason):
        pass

    def scale_cluster(self, cluster, node_group_id_map):
        pass

    def shutdown_cluster(self, cluster):
        pass


class TestEngine(base.SaharaWithDbTestCase):

    def setUp(self):
        super(TestEngine, self).setUp()
        self.eng = EngineTest()

    @mock.patch('sahara.utils.openstack.images.SaharaImageManager')
    def test_get_node_group_image_username(self, mock_manager):
        ng = mock.Mock()
        manager = mock.Mock()
        manager.get.return_value = mock.Mock(username='username')
        mock_manager.return_value = manager

        self.assertEqual(
            'username', self.eng.get_node_group_image_username(ng))

    @mock.patch('sahara.utils.cluster_progress_ops.add_successful_event')
    @mock.patch('sahara.service.networks.init_instances_ips',
                return_value=True)
    @mock.patch('sahara.context.set_current_instance_id')
    @mock.patch('sahara.utils.cluster.check_cluster_exists', return_value=True)
    def test_ips_assign(self, g, ctx, init, ops):
        cluster = mock.Mock()
        instances = [mock.Mock(id='1'), mock.Mock(id='2')]
        instances_with_ip = set()
        self.eng._ips_assign(instances_with_ip, cluster, instances)
        self.assertEqual({'1', '2'}, instances_with_ip)

    @mock.patch('datetime.datetime')
    @mock.patch('sahara.context.ctx')
    @mock.patch('sahara.service.engine.conductor')
    def test_clean_job_executions(self, conductor, ctx, date):
        cluster = mock.Mock()
        je = mock.Mock(info=None, end_time=None)
        conductor.job_execution_get_all.return_value = [je]
        date.now.return_value = '28.04.2015'

        self.eng._clean_job_executions(cluster)

        args, kwargs = conductor.job_execution_update.call_args
        update = {
            'info': {'status': 'KILLED'},
            'cluster_id': None,
            'end_time': '28.04.2015'}

        self.assertEqual(update, args[2])


class TestDeletion(base.SaharaTestCase):
    def setUp(self):
        super(TestDeletion, self).setUp()
        self.engine = EngineTest()

    @mock.patch('sahara.service.engine.Engine._remove_db_objects')
    @mock.patch('sahara.service.engine.Engine._clean_job_executions')
    @mock.patch('sahara.utils.openstack.heat.client')
    @mock.patch('sahara.service.heat.heat_engine.LOG.warning')
    def test_calls_order(self, logger, heat_client, _job_ex, _db_ob):
        class FakeHeatEngine(heat_engine.HeatEngine):
            def __init__(self):
                super(FakeHeatEngine, self).__init__()
                self.order = []

            def _clean_job_executions(self, cluster):
                self.order.append('clean_job_executions')
                super(FakeHeatEngine, self)._clean_job_executions(cluster)

            def _remove_db_objects(self, cluster):
                self.order.append('remove_db_objects')
                super(FakeHeatEngine, self)._remove_db_objects(cluster)

        fake_cluster = mock.Mock()
        heat_client.side_effect = heat_exc.HTTPNotFound()
        engine = FakeHeatEngine()
        engine.shutdown_cluster(fake_cluster)
        self.assertEqual(['clean_job_executions', 'remove_db_objects'],
                         engine.order)
        self.assertEqual(
            [mock.call('Did not find stack for cluster.')],
            logger.call_args_list
        )

    @mock.patch('sahara.service.heat.heat_engine.LOG.error')
    @mock.patch('sahara.utils.openstack.heat.delete_stack')
    @mock.patch('sahara.utils.openstack.heat.lazy_delete_stack')
    @mock.patch('sahara.service.engine.Engine._remove_db_objects')
    @mock.patch('sahara.service.engine.Engine._clean_job_executions')
    def test_force_delete_calls(self, cj, rdb, lazy_delete, delete, log_err):
        engine = heat_engine.HeatEngine()

        # Force delete (lazy_delete)
        engine.shutdown_cluster(mock.Mock(), force=True)
        self.assertEqual(delete.call_count, 0)
        self.assertEqual(lazy_delete.call_count, 1)
        self.assertEqual(cj.call_count, 1)
        self.assertEqual(rdb.call_count, 1)

        delete.reset_mock()
        lazy_delete.reset_mock()
        rdb.reset_mock()
        cj.reset_mock()

        # Regular delete
        engine.shutdown_cluster(mock.Mock(), force=False)
        self.assertEqual(delete.call_count, 1)
        self.assertEqual(lazy_delete.call_count, 0)
        self.assertEqual(cj.call_count, 1)
        self.assertEqual(rdb.call_count, 1)



# Copyright (c) 2016 SUSE LINUX Products GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import testtools

from sahara import context
from sahara import main


def mock_validate_config():
    return mock.patch(
        'sahara.service.castellan.config.validate_config')


class SomeException(Exception):
    pass


class ValidateCastellanTest(testtools.TestCase):

    def test_castellan_validate_config_called(self):
        with mock_validate_config() as validate_config:
            main.validate_castellan_config()

        validate_config.assert_called_once_with()

    def test_context_present_when_calling_validate_config(self):

        def check_context():
            self.assertTrue(context.has_ctx())

        with mock_validate_config() as validate_config:
            validate_config.side_effect = check_context
            main.validate_castellan_config()

    def test_context_cleared(self):
        with mock_validate_config():
            main.validate_castellan_config()

        self.assertFalse(context.has_ctx())

    def test_context_cleared_in_case_of_exception(self):
        with mock_validate_config() as validate_config:
            validate_config.side_effect = SomeException
            self.assertRaises(SomeException, main.validate_castellan_config)

        self.assertFalse(context.has_ctx())



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Handles all requests to the conductor service."""

from oslo_config import cfg

from sahara.conductor import manager
from sahara.conductor import resource as r


conductor_opts = [
    cfg.BoolOpt('use_local',
                default=True,
                help='Perform sahara-conductor operations locally.'),
]

conductor_group = cfg.OptGroup(name='conductor',
                               title='Conductor Options')

CONF = cfg.CONF
CONF.register_group(conductor_group)
CONF.register_opts(conductor_opts, conductor_group)


def _get_id(obj):
    """Return object id.

    Allows usage of both an object or an object's ID as a parameter when
    dealing with relationships.
    """
    try:
        return obj.id
    except AttributeError:
        return obj


class LocalApi(object):
    """A local version of the conductor API.

    It does database updates locally instead of via RPC.
    """

    def __init__(self):
        self._manager = manager.ConductorManager()

    # Cluster ops

    @r.wrap(r.ClusterResource)
    def cluster_get(self, context, cluster, show_progress=False):
        """Return the cluster or None if it does not exist."""
        return self._manager.cluster_get(
            context, _get_id(cluster), show_progress)

    @r.wrap(r.ClusterResource)
    def cluster_get_all(self, context, regex_search=False, **kwargs):
        """Get all clusters filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self._manager.cluster_get_all(context, regex_search, **kwargs)

    @r.wrap(r.ClusterResource)
    def cluster_create(self, context, values):
        """Create a cluster from the values dictionary.

        :returns: the created cluster.
        """
        return self._manager.cluster_create(context, values)

    @r.wrap(r.ClusterResource)
    def cluster_update(self, context, cluster, values):
        """Update the cluster with the given values dictionary.

        :returns: the updated cluster.
        """
        return self._manager.cluster_update(context, _get_id(cluster),
                                            values)

    def cluster_destroy(self, context, cluster):
        """Destroy the cluster or raise if it does not exist.

        :returns: None.
        """
        self._manager.cluster_destroy(context, _get_id(cluster))

    # Node Group ops

    def node_group_add(self, context, cluster, values):
        """Create a node group from the values dictionary.

        :returns: ID of the created node group.
        """
        return self._manager.node_group_add(context, _get_id(cluster), values)

    def node_group_update(self, context, node_group, values):
        """Update the node group with the given values dictionary.

        :returns: None.
        """
        self._manager.node_group_update(context, _get_id(node_group), values)

    def node_group_remove(self, context, node_group):
        """Destroy the node group or raise if it does not exist.

        :returns: None.
        """
        self._manager.node_group_remove(context, _get_id(node_group))

    # Instance ops

    def instance_add(self, context, node_group, values):
        """Create an instance from the values dictionary.

        :returns: ID of the created instance.
        """
        return self._manager.instance_add(context, _get_id(node_group), values)

    def instance_update(self, context, instance, values):
        """Update the instance with the given values dictionary.

        :returns: None.
        """
        self._manager.instance_update(context, _get_id(instance), values)

    def instance_remove(self, context, instance):
        """Destroy the instance or raise if it does not exist.

        :returns: None.
        """
        self._manager.instance_remove(context, _get_id(instance))

    # Volumes ops

    def append_volume(self, context, instance, volume_id):
        """Append volume_id to instance."""
        self._manager.append_volume(context, _get_id(instance), volume_id)

    def remove_volume(self, context, instance, volume_id):
        """Remove volume_id in instance."""
        self._manager.remove_volume(context, _get_id(instance), volume_id)

    # Cluster Template ops

    @r.wrap(r.ClusterTemplateResource)
    def cluster_template_get(self, context, cluster_template):
        """Return the cluster template or None if it does not exist."""
        return self._manager.cluster_template_get(context,
                                                  _get_id(cluster_template))

    @r.wrap(r.ClusterTemplateResource)
    def cluster_template_get_all(self, context, regex_search=False, **kwargs):
        """Get all cluster templates filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self._manager.cluster_template_get_all(context,
                                                      regex_search, **kwargs)

    @r.wrap(r.ClusterTemplateResource)
    def cluster_template_create(self, context, values):
        """Create a cluster template from the values dictionary.

        :returns: the created cluster template
        """
        return self._manager.cluster_template_create(context, values)

    def cluster_template_destroy(self, context, cluster_template,
                                 ignore_prot_on_def=False):
        """Destroy the cluster template or raise if it does not exist.

        :returns: None
        """
        self._manager.cluster_template_destroy(context,
                                               _get_id(cluster_template),
                                               ignore_prot_on_def)

    @r.wrap(r.ClusterTemplateResource)
    def cluster_template_update(self, context, id, cluster_template,
                                ignore_prot_on_def=False):
        """Update the cluster template or raise if it does not exist.

        :returns: the updated cluster template
        """
        return self._manager.cluster_template_update(context,
                                                     id,
                                                     cluster_template,
                                                     ignore_prot_on_def)

    # Node Group Template ops

    @r.wrap(r.NodeGroupTemplateResource)
    def node_group_template_get(self, context, node_group_template):
        """Return the node group template or None if it does not exist."""
        return self._manager.node_group_template_get(
            context, _get_id(node_group_template))

    @r.wrap(r.NodeGroupTemplateResource)
    def node_group_template_get_all(self,
                                    context, regex_search=False, **kwargs):
        """Get all node group templates filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self._manager.node_group_template_get_all(
            context, regex_search, **kwargs)

    @r.wrap(r.NodeGroupTemplateResource)
    def node_group_template_create(self, context, values):
        """Create a node group template from the values dictionary.

        :returns: the created node group template
        """
        return self._manager.node_group_template_create(context, values)

    def node_group_template_destroy(self, context, node_group_template,
                                    ignore_prot_on_def=False):
        """Destroy the node group template or raise if it does not exist.

        :returns: None
        """
        self._manager.node_group_template_destroy(context,
                                                  _get_id(node_group_template),
                                                  ignore_prot_on_def)

    @r.wrap(r.NodeGroupTemplateResource)
    def node_group_template_update(self, context, id, values,
                                   ignore_prot_on_def=False):
        """Update a node group template from the values dictionary.

        :returns: the updated node group template
        """
        return self._manager.node_group_template_update(context, id, values,
                                                        ignore_prot_on_def)

    # Data Source ops

    @r.wrap(r.DataSource)
    def data_source_get(self, context, data_source):
        """Return the Data Source or None if it does not exist."""
        return self._manager.data_source_get(context, _get_id(data_source))

    @r.wrap(r.DataSource)
    def data_source_get_all(self, context, regex_search=False, **kwargs):
        """Get all Data Sources filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self._manager.data_source_get_all(context,
                                                 regex_search, **kwargs)

    def data_source_count(self, context, **kwargs):
        """Count Data Sources filtered by **kwargs.

        Uses sqlalchemy "in_" clause for any tuple values
        Uses sqlalchemy "like" clause for any string values containing %
        """
        return self._manager.data_source_count(context, **kwargs)

    @r.wrap(r.DataSource)
    def data_source_create(self, context, values):
        """Create a Data Source from the values dictionary."""
        return self._manager.data_source_create(context, values)

    def data_source_destroy(self, context, data_source):
        """Destroy the Data Source or raise if it does not exist."""
        self._manager.data_source_destroy(context, _get_id(data_source))

    @r.wrap(r.DataSource)
    def data_source_update(self, context, id, values):
        """Update an existing Data Source"""
        return self._manager.data_source_update(context, id, values)

    # JobExecution ops

    @r.wrap(r.JobExecution)
    def job_execution_get(self, context, job_execution):
        """Return the JobExecution or None if it does not exist."""
        return self._manager.job_execution_get(context,
                                               _get_id(job_execution))

    @r.wrap(r.JobExecution)
    def job_execution_get_all(self, context, regex_search=False, **kwargs):
        """Get all JobExecutions filtered by **kwargs.

        kwargs key values may be the names of fields in a JobExecution
        plus the following special values with the indicated meaning:

        'cluster.name' -- name of the Cluster referenced by the JobExecution
        'job.name' -- name of the Job referenced by the JobExecution
        'status' -- JobExecution['info']['status']

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self._manager.job_execution_get_all(context,
                                                   regex_search, **kwargs)

    def job_execution_count(self, context, **kwargs):
        """Count number of JobExecutions filtered by **kwargs.

        e.g. job_execution_count(cluster_id=12, input_id=123)
        """
        return self._manager.job_execution_count(context, **kwargs)

    @r.wrap(r.JobExecution)
    def job_execution_create(self, context, values):
        """Create a JobExecution from the values dictionary."""
        return self._manager.job_execution_create(context, values)

    @r.wrap(r.JobExecution)
    def job_execution_update(self, context, job_execution, values):
        """Update the JobExecution or raise if it does not exist."""
        return self._manager.job_execution_update(context,
                                                  _get_id(job_execution),
                                                  values)

    def job_execution_destroy(self, context, job_execution):
        """Destroy the JobExecution or raise if it does not exist."""
        self._manager.job_execution_destroy(context, _get_id(job_execution))

    # Job ops

    @r.wrap(r.Job)
    def job_get(self, context, job):
        """Return the Job or None if it does not exist."""
        return self._manager.job_get(context, _get_id(job))

    @r.wrap(r.Job)
    def job_get_all(self, context, regex_search=False, **kwargs):
        """Get all Jobs filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self._manager.job_get_all(context, regex_search, **kwargs)

    @r.wrap(r.Job)
    def job_create(self, context, values):
        """Create a Job from the values dictionary."""
        return self._manager.job_create(context, values)

    @r.wrap(r.Job)
    def job_update(self, context, job, values):
        """Update the Job or raise if it does not exist."""
        return self._manager.job_update(context, _get_id(job),
                                        values)

    def job_destroy(self, context, job):
        """Destroy the Job or raise if it does not exist."""
        self._manager.job_destroy(context, _get_id(job))

    def job_main_name(self, context, job):
        """Return the name of the first main JobBinary or None.

        At present the 'mains' element is expected to contain a single element.
        In the future if 'mains' contains more than one element we will need
        a scheme or convention for retrieving a name from the list of binaries.

        :param job: This is expected to be a Job object
        """
        if job.mains:
            binary = self.job_binary_get(context, job.mains[0])
            if binary is not None:
                return binary["name"]
        return None

    def job_lib_names(self, context, job):
        """Return the name of all job lib binaries or an empty list.

        :param job: This is expected to be a Job object
        """
        lib_ids = job.libs or []
        binaries = (self.job_binary_get(context, lib_id) for lib_id in lib_ids)
        return [binary["name"] for binary in binaries if binary is not None]

    # JobBinary ops

    @r.wrap(r.JobBinary)
    def job_binary_get_all(self, context, regex_search=False, **kwargs):
        """Get all JobBinarys filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self._manager.job_binary_get_all(context,
                                                regex_search, **kwargs)

    @r.wrap(r.JobBinary)
    def job_binary_get(self, context, job_binary):
        """Return the JobBinary or None if it does not exist."""
        return self._manager.job_binary_get(context, _get_id(job_binary))

    @r.wrap(r.JobBinary)
    def job_binary_create(self, context, values):
        """Create a JobBinary from the values dictionary."""
        return self._manager.job_binary_create(context, values)

    def job_binary_destroy(self, context, job_binary):
        """Destroy the JobBinary or raise if it does not exist."""
        self._manager.job_binary_destroy(context, _get_id(job_binary))

    @r.wrap(r.JobBinary)
    def job_binary_update(self, context, id, values):
        """Update a JobBinary from the values dictionary."""
        return self._manager.job_binary_update(context, id, values)

    # JobBinaryInternal ops

    @r.wrap(r.JobBinaryInternal)
    def job_binary_internal_get_all(self, context,
                                    regex_search=False, **kwargs):
        """Get all JobBinaryInternals filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self._manager.job_binary_internal_get_all(
            context, regex_search, **kwargs)

    @r.wrap(r.JobBinaryInternal)
    def job_binary_internal_get(self, context, job_binary_internal):
        """Return the JobBinaryInternal or None if it does not exist."""
        return self._manager.job_binary_internal_get(
            context,
            _get_id(job_binary_internal))

    @r.wrap(r.JobBinaryInternal)
    def job_binary_internal_create(self, context, values):
        """Create a JobBinaryInternal from the values dictionary."""
        return self._manager.job_binary_internal_create(context, values)

    def job_binary_internal_destroy(self, context, job_binary_internal_id):
        """Destroy the JobBinaryInternal or raise if it does not exist."""
        self._manager.job_binary_internal_destroy(
            context,
            _get_id(job_binary_internal_id))

    def job_binary_internal_get_raw_data(self, context,
                                         job_binary_internal_id):
        """Return the binary data field from a JobBinaryInternal."""
        return self._manager.job_binary_internal_get_raw_data(
            context,
            job_binary_internal_id)

    @r.wrap(r.JobBinaryInternal)
    def job_binary_internal_update(self, context, job_binary_internal, values):
        """Update a JobBinaryInternal from the values dictionary."""
        return self._manager.job_binary_internal_update(
            context, _get_id(job_binary_internal), values)

    # Events ops

    def cluster_provision_step_add(self, context, cluster_id, values):
        """Create a provisioning step assigned to cluster from values dict."""
        return self._manager.cluster_provision_step_add(
            context, cluster_id, values)

    def cluster_provision_step_update(self, context, provision_step):
        """Update the cluster provisioning step."""
        return self._manager.cluster_provision_step_update(
            context, provision_step)

    def cluster_provision_progress_update(self, context, cluster_id):
        """Return cluster with provision progress updated field."""
        return self._manager.cluster_provision_progress_update(
            context, cluster_id)

    def cluster_event_add(self, context, provision_step, values):
        """Assign new event to the specified provision step."""
        return self._manager.cluster_event_add(
            context, provision_step, values)

    @r.wrap(r.ClusterVerificationResource)
    def cluster_verification_add(self, context, cluster_id, values):
        """Return created verification for the specified cluster."""
        return self._manager.cluster_verification_add(
            context, _get_id(cluster_id), values)

    @r.wrap(r.ClusterVerificationResource)
    def cluster_verification_get(self, context, verification_id):
        """Return verification with the specified verification_id."""
        return self._manager.cluster_verification_get(
            context, _get_id(verification_id))

    @r.wrap(r.ClusterVerificationResource)
    def cluster_verification_update(self, context, verification_id, values):
        """Return updated verification with the specified verification_id."""
        return self._manager.cluster_verification_update(
            context, _get_id(verification_id), values)

    def cluster_verification_delete(self, context, verification_id):
        """"Delete verification with the specified id."""
        return self._manager.cluster_verification_delete(
            context, _get_id(verification_id))

    @r.wrap(r.ClusterHealthCheckResource)
    def cluster_health_check_add(self, context, verification_id, values):
        """Return created health check in the specified verification."""
        return self._manager.cluster_health_check_add(
            context, _get_id(verification_id), values)

    @r.wrap(r.ClusterHealthCheckResource)
    def cluster_health_check_get(self, context, health_check_id):
        """Return health check with the specified health_check_id."""
        return self._manager.cluster_health_check_get(
            context, _get_id(health_check_id))

    @r.wrap(r.ClusterHealthCheckResource)
    def cluster_health_check_update(self, context, health_check_id, values):
        """Return updated health check with the specified health_check_id."""
        return self._manager.cluster_health_check_update(
            context, _get_id(health_check_id), values)

    def plugin_create(self, context, values):
        """Return created DB entry for plugin."""
        return self._manager.plugin_create(context, values)

    def plugin_get(self, context, name):
        """Return DB entry for plugin."""
        return self._manager.plugin_get(context, name)

    def plugin_get_all(self, context):
        """Return DB entries for all plugins."""
        return self._manager.plugin_get_all(context)

    def plugin_update(self, context, name, values):
        """Return updated DB entry for plugin."""
        return self._manager.plugin_update(context, name, values)

    def plugin_remove(self, context, name):
        """Remove DB entry for plugin."""
        return self._manager.plugin_remove(context, name)


class RemoteApi(LocalApi):
    """Conductor API that does updates via RPC to the ConductorManager."""

    # TODO(slukjanov): it should override _manager and only necessary functions



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from sahara.conductor import manager
from sahara import context
import sahara.tests.unit.conductor.base as test_base
from sahara.tests.unit.conductor.manager import test_clusters
from sahara.tests.unit.conductor.manager import test_templates


CORRECT_CONF = {
    u'service_1': {u'config_2': u'value_2', u'config_1': u'value_1'},
    u'service_2': {u'config_1': u'value_1'}
}


class ObjectsFromTemplatesTest(test_base.ConductorManagerTestCase):
    def __init__(self, *args, **kwargs):
        super(ObjectsFromTemplatesTest, self).__init__(
            checks=[
                lambda: CORRECT_CONF,
                lambda: test_clusters.SAMPLE_CLUSTER,
                lambda: test_templates.SAMPLE_CLT,
                lambda: test_templates.SAMPLE_NGT,
                lambda: manager.CLUSTER_DEFAULTS,
                lambda: manager.NODE_GROUP_DEFAULTS,
                lambda: manager.INSTANCE_DEFAULTS,
            ], *args, **kwargs)

    def test_cluster_create_from_templates(self):
        ctx = context.ctx()

        # create node_group_template
        ng_tmpl = copy.deepcopy(test_templates.SAMPLE_NGT)
        ng_tmpl['volumes_size'] = 10
        ng_tmpl['node_configs']['service_1']['config_2'] = 'value_2'
        ng_tmpl = self.api.node_group_template_create(ctx, ng_tmpl)

        # create cluster template
        cl_tmpl = self.api.cluster_template_create(ctx,
                                                   test_templates.SAMPLE_CLT)

        # create cluster
        cluster_val = copy.deepcopy(test_clusters.SAMPLE_CLUSTER)
        cluster_val['cluster_template_id'] = cl_tmpl['id']
        cluster_val['node_groups'][0]['node_group_template_id'] = ng_tmpl['id']
        cluster = self.api.cluster_create(ctx, cluster_val)
        self.assertEqual(CORRECT_CONF, cluster['cluster_configs'])

        for node_group in cluster['node_groups']:
            if node_group['name'] == 'ng_1':
                self.assertEqual(['p1', 'p2'], node_group['node_processes'])
                self.assertEqual(10, node_group['volumes_size'])
                self.assertEqual(CORRECT_CONF, node_group['node_configs'])

    def test_node_group_add_from_template(self):
        ctx = context.ctx()

        # create cluster
        sample_copy = copy.deepcopy(test_clusters.SAMPLE_CLUSTER)
        cluster = self.api.cluster_create(ctx, sample_copy)

        # create node_group_template
        ng_tmpl = copy.deepcopy(test_templates.SAMPLE_NGT)
        ng_tmpl['volumes_size'] = 10
        ng_tmpl['node_configs']['service_1']['config_2'] = 'value_2'
        ng_tmpl = self.api.node_group_template_create(ctx, ng_tmpl)

        # add node group to cluster
        ng = copy.deepcopy(test_clusters.SAMPLE_CLUSTER['node_groups'][0])
        ng['node_group_template_id'] = ng_tmpl['id']
        ng['count'] = 5
        ng['name'] = 'ng_3'
        self.api.node_group_add(ctx, cluster['id'], ng)

        # refetch cluster
        cluster = self.api.cluster_get(ctx, cluster['id'])

        for node_group in cluster['node_groups']:
            if node_group['name'] == 'ng_3':
                self.assertEqual(['p1', 'p2'], node_group['node_processes'])
                self.assertEqual(10, node_group['volumes_size'])
                self.assertEqual(CORRECT_CONF, node_group['node_configs'])
                self.assertEqual(5, node_group['count'])



# Copyright (c) 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara import context


def ctx(**kwargs):
    return context.ctx()


def set_ctx(new_ctx, **kwargs):
    context.set_ctx(new_ctx)


def has_ctx(**kwargs):
    return context.has_ctx()


def sleep(seconds=0, **kwargs):
    context.sleep(seconds)


def current(**kwargs):
    return context.current()


def set_current_instance_id(instance_id, **kwargs):
    return context.set_current_instance_id(instance_id)


class PluginsThreadGroup(context.ThreadGroup):

    def __init__(self, thread_pool_size=1000, **kwargs):
        super(PluginsThreadGroup, self).__init__()


class PluginsContext(context.Context):

    def __init__(self,
                 user_id=None,
                 tenant_id=None,
                 auth_token=None,
                 service_catalog=None,
                 username=None,
                 tenant_name=None,
                 roles=None,
                 is_admin=None,
                 remote_semaphore=None,
                 resource_uuid=None,
                 current_instance_info=None,
                 request_id=None,
                 auth_plugin=None,
                 overwrite=True,
                 **kwargs):

        super(PluginsContext, self).__init__(auth_token=auth_token,
                                             user=user_id,
                                             tenant=tenant_id,
                                             is_admin=is_admin,
                                             resource_uuid=resource_uuid,
                                             request_id=request_id,
                                             roles=roles, **kwargs)



# Copyright (c) 2018 OpenStack Contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six.moves.urllib.parse as urlparse

from sahara import exceptions as ex
from sahara.i18n import _
from sahara.service.edp.data_sources.base import DataSourceType
from sahara.service.edp import s3_common
from sahara.utils import types


class S3Type(DataSourceType):
    configs_map = {"accesskey": s3_common.S3_ACCESS_KEY_CONFIG,
                   "secretkey": s3_common.S3_SECRET_KEY_CONFIG,
                   "endpoint": s3_common.S3_ENDPOINT_CONFIG,
                   "bucket_in_path": s3_common.S3_BUCKET_IN_PATH_CONFIG,
                   "ssl": s3_common.S3_SSL_CONFIG}

    bool_keys = ["bucket_in_path",
                 "ssl"]

    def validate(self, data):
        self._validate_url(data['url'])

        # Do validation loosely, and don't require much... the user might have
        # (by their own preference) set some or all configs manually

        if "credentials" not in data:
            return

        for key in data["credentials"].keys():
            if key not in self.configs_map.keys():
                raise ex.InvalidDataException(
                    _("Unknown config '%s' for S3 data source") % key)
            if key in self.bool_keys:
                if not isinstance(data["credentials"][key], bool):
                    raise ex.InvalidDataException(
                        _("Config '%s' must be boolean") % key)

    def _validate_url(self, url):
        if len(url) == 0:
            raise ex.InvalidDataException(_("S3 url must not be empty"))

        url = urlparse.urlparse(url)
        if url.scheme not in ["s3", "s3a"]:
            raise ex.InvalidDataException(
                _("URL scheme must be 's3' or 's3a'"))

        if not url.hostname:
            raise ex.InvalidDataException(_("Bucket name must be present"))

        if not url.path:
            raise ex.InvalidDataException(_("Object name must be present"))

    def prepare_cluster(self, data_source, cluster, **kwargs):
        if hasattr(data_source, "credentials"):
            job_configs = kwargs.pop('job_configs')

            if isinstance(job_configs, types.FrozenDict):
                return
            if job_configs.get('configs', None) is None:
                return

            creds = data_source.credentials
            job_conf = job_configs['configs']

            for config_name, s3a_cfg_name in self.configs_map.items():
                if job_conf.get(s3a_cfg_name, None) is None:  # no overwrite
                    if creds.get(config_name, None) is not None:
                        job_conf[s3a_cfg_name] = creds[config_name]

    def get_runtime_url(self, url, cluster):
        return url.replace("s3://", "s3a://", 1)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_policy import policy

DATA_PROCESSING = 'data-processing:%s'
DATA_PROCESSING_CLUSTERS = DATA_PROCESSING % 'clusters:%s'
DATA_PROCESSING_CLUSTER_TEMPLATES = DATA_PROCESSING % 'cluster-templates:%s'
DATA_PROCESSING_DATA_SOURCES = DATA_PROCESSING % 'data-sources:%s'
DATA_PROCESSING_IMAGES = DATA_PROCESSING % 'images:%s'
DATA_PROCESSING_JOB_BINARIES = DATA_PROCESSING % 'job-binaries:%s'
DATA_PROCESSING_JOB_EXECUTIONS = DATA_PROCESSING % 'job-executions:%s'
DATA_PROCESSING_JOB_TYPES = DATA_PROCESSING % 'job-types:%s'
DATA_PROCESSING_JOBS = DATA_PROCESSING % 'jobs:%s'
DATA_PROCESSING_PLUGINS = DATA_PROCESSING % 'plugins:%s'
DATA_PROCESSING_NODE_GROUP_TEMPLATES = (
    DATA_PROCESSING % 'node-group-templates:%s')
DATA_PROCESSING_JOB_BINARY_INTERNALS = (
    DATA_PROCESSING % 'job-binary-internals:%s')

DATA_PROCESSING_CLUSTER = DATA_PROCESSING % 'cluster:%s'
DATA_PROCESSING_CLUSTER_TEMPLATE = DATA_PROCESSING % 'cluster-template:%s'
DATA_PROCESSING_DATA_SOURCE = DATA_PROCESSING % 'data-source:%s'
DATA_PROCESSING_IMAGE = DATA_PROCESSING % 'image:%s'
DATA_PROCESSING_JOB_BINARY = DATA_PROCESSING % 'job-binary:%s'
DATA_PROCESSING_JOB_TEMPLATE = DATA_PROCESSING % 'job-template:%s'
DATA_PROCESSING_JOB_TYPE = DATA_PROCESSING % 'job-type:%s'
DATA_PROCESSING_JOB = DATA_PROCESSING % 'job:%s'
DATA_PROCESSING_PLUGIN = DATA_PROCESSING % 'plugin:%s'
DATA_PROCESSING_NODE_GROUP_TEMPLATE = (
    DATA_PROCESSING % 'node-group-template:%s')
UNPROTECTED = ''
ROLE_ADMIN = 'role:admin'

rules = [
    policy.RuleDefault(
        name='context_is_admin',
        check_str=ROLE_ADMIN),
]


def list_rules():
    return rules



# Copyright (c) 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
import os
import xml.dom.minidom as xml

from oslo_config import cfg
from oslo_utils import uuidutils
import six

from sahara import conductor as c
from sahara import context
from sahara.service.edp import base_engine
from sahara.service.edp import hdfs_helper as h
from sahara.service.edp.job_binaries import manager as jb_manager
from sahara.service.edp import job_utils
from sahara.service.edp.oozie import oozie as o
from sahara.service.edp.oozie.workflow_creator import workflow_factory
from sahara.service.validations.edp import job_execution as j
from sahara.utils import edp
from sahara.utils import remote
from sahara.utils import xmlutils as x


CONF = cfg.CONF

conductor = c.API


@six.add_metaclass(abc.ABCMeta)
class OozieJobEngine(base_engine.JobEngine):

    def __init__(self, cluster):
        self.cluster = cluster
        self.plugin = job_utils.get_plugin(self.cluster)

    def get_remote_client(self):
        return o.RemoteOozieClient(self.get_oozie_server_uri(self.cluster),
                                   self.get_oozie_server(self.cluster),
                                   self.get_hdfs_user())

    def get_client(self):
        # by default engine will return standard oozie client implementation
        return o.OozieClient(self.get_oozie_server_uri(self.cluster),
                             self.get_oozie_server(self.cluster))

    def _get_oozie_job_params(self, hdfs_user, path_to_workflow,
                              oozie_params, use_hbase_lib,
                              scheduled_params=None, job_dir=None,
                              job_execution_type=None):
        oozie_libpath_key = "oozie.libpath"
        oozie_libpath = ""
        rm_path = self.get_resource_manager_uri(self.cluster)
        nn_path = self.get_name_node_uri(self.cluster)
        hbase_common_lib_path = "%s%s" % (nn_path, h.HBASE_COMMON_LIB_PATH)

        if use_hbase_lib:
            if oozie_libpath_key in oozie_params:
                oozie_libpath = "%s,%s" % (oozie_params.get(oozie_libpath_key,
                                           ""), hbase_common_lib_path)
            else:
                oozie_libpath = hbase_common_lib_path

        if job_execution_type == "scheduled":
            app_path = "oozie.coord.application.path"
            job_parameters = {
                "start": scheduled_params.get('start'),
                "end": scheduled_params.get('end'),
                "frequency": scheduled_params.get('frequency'),
                "workflowAppUri": "%s%s" % (nn_path, job_dir),
                app_path: "%s%s" % (nn_path, job_dir)}
        else:
            app_path = "oozie.wf.application.path"
            job_parameters = {
                app_path: "%s%s" % (nn_path, path_to_workflow)}

        job_parameters["nameNode"] = nn_path
        job_parameters["user.name"] = hdfs_user
        job_parameters["jobTracker"] = rm_path
        job_parameters[oozie_libpath_key] = oozie_libpath
        job_parameters["oozie.use.system.libpath"] = "true"

        # Don't let the application path be overwritten, that can't
        # possibly make any sense
        if app_path in oozie_params:
            del oozie_params[app_path]
        if oozie_libpath_key in oozie_params:
            del oozie_params[oozie_libpath_key]

        job_parameters.update(oozie_params)
        return job_parameters

    def _upload_workflow_file(self, where, job_dir, wf_xml, hdfs_user):
        with remote.get_remote(where) as r:
            h.put_file_to_hdfs(r, wf_xml, "workflow.xml", job_dir, hdfs_user)
        return "%s/workflow.xml" % job_dir

    def _upload_coordinator_file(self, where, job_dir, wf_xml, hdfs_user):
        with remote.get_remote(where) as r:
            h.put_file_to_hdfs(r, wf_xml, "coordinator.xml", job_dir,
                               hdfs_user)
        return "%s/coordinator.xml" % job_dir

    def cancel_job(self, job_execution):
        if job_execution.engine_job_id is not None:
            client = self.get_client()
            client.kill_job(job_execution)
            return client.get_job_info(job_execution)

    def get_job_status(self, job_execution):
        if job_execution.engine_job_id is not None:
            return self.get_client().get_job_info(job_execution)

    def _prepare_run_job(self, job_execution):
        ctx = context.ctx()

        # This will be a dictionary of tuples, (native_url, runtime_url)
        # keyed by data_source id
        data_source_urls = {}

        prepared_job_params = {}

        job = conductor.job_get(ctx, job_execution.job_id)

        input_source, output_source = job_utils.get_input_output_data_sources(
            job_execution, job, data_source_urls, self.cluster)

        # Updated_job_configs will be a copy of job_execution.job_configs with
        # any name or uuid references to data_sources resolved to paths
        # assuming substitution is enabled.
        # If substitution is not enabled then updated_job_configs will
        # just be a reference to job_execution.job_configs to avoid a copy.
        # Additional_sources will be a list of any data_sources found.
        additional_sources, updated_job_configs = (
            job_utils.resolve_data_source_references(job_execution.job_configs,
                                                     job_execution.id,
                                                     data_source_urls,
                                                     self.cluster)
        )

        job_execution = conductor.job_execution_update(
            ctx, job_execution,
            {"data_source_urls": job_utils.to_url_dict(data_source_urls)})

        # Now that we've recorded the native urls, we can switch to the
        # runtime urls
        data_source_urls = job_utils.to_url_dict(data_source_urls,
                                                 runtime=True)

        data_sources = additional_sources + [input_source, output_source]
        job_utils.prepare_cluster_for_ds(data_sources,
                                         self.cluster, updated_job_configs,
                                         data_source_urls)

        proxy_configs = updated_job_configs.get('proxy_configs')
        configs = updated_job_configs.get('configs', {})
        use_hbase_lib = configs.get('edp.hbase_common_lib', {})

        # Extract all the 'oozie.' configs so that they can be set in the
        # job properties file. These are config values for Oozie itself,
        # not the job code
        oozie_params = {}
        for k in list(configs):
            if k.startswith('oozie.'):
                oozie_params[k] = configs[k]

        external_hdfs_urls = self._resolve_external_hdfs_urls(
            job_execution.job_configs)
        for url in external_hdfs_urls:
            h.configure_cluster_for_hdfs(self.cluster, url)

        hdfs_user = self.get_hdfs_user()

        # TODO(tmckay): this should probably be "get_namenode"
        # but that call does not exist in the oozie engine api now.
        oozie_server = self.get_oozie_server(self.cluster)

        wf_dir = self._create_hdfs_workflow_dir(oozie_server, job)
        self._upload_job_files_to_hdfs(oozie_server, wf_dir, job, configs,
                                       proxy_configs)

        wf_xml = workflow_factory.get_workflow_xml(
            job, self.cluster, updated_job_configs,
            input_source, output_source,
            hdfs_user, data_source_urls)

        path_to_workflow = self._upload_workflow_file(oozie_server, wf_dir,
                                                      wf_xml, hdfs_user)

        prepared_job_params['context'] = ctx
        prepared_job_params['hdfs_user'] = hdfs_user
        prepared_job_params['path_to_workflow'] = path_to_workflow
        prepared_job_params['use_hbase_lib'] = use_hbase_lib
        prepared_job_params['job_execution'] = job_execution
        prepared_job_params['oozie_params'] = oozie_params
        prepared_job_params['wf_dir'] = wf_dir
        prepared_job_params['oozie_server'] = oozie_server

        return prepared_job_params

    def run_job(self, job_execution):
        prepared_job_params = self._prepare_run_job(job_execution)
        path_to_workflow = prepared_job_params['path_to_workflow']
        hdfs_user = prepared_job_params['hdfs_user']
        oozie_params = prepared_job_params['oozie_params']
        use_hbase_lib = prepared_job_params['use_hbase_lib']
        ctx = prepared_job_params['context']
        job_execution = prepared_job_params['job_execution']

        job_params = self._get_oozie_job_params(hdfs_user,
                                                path_to_workflow,
                                                oozie_params,
                                                use_hbase_lib)

        client = self.get_client()
        oozie_job_id = client.add_job(x.create_hadoop_xml(job_params),
                                      job_execution)
        job_execution = conductor.job_execution_get(ctx, job_execution.id)
        if job_execution.info['status'] == edp.JOB_STATUS_TOBEKILLED:
            return (None, edp.JOB_STATUS_KILLED, None)

        conductor.job_execution_update(
            context.ctx(), job_execution.id,
            {'info': {'status': edp.JOB_STATUS_READYTORUN},
             'engine_job_id': oozie_job_id})

        client.run_job(job_execution, oozie_job_id)
        try:
            status = client.get_job_info(job_execution, oozie_job_id)['status']
        except Exception:
            status = None
        return (oozie_job_id, status, None)

    def run_scheduled_job(self, job_execution):
        prepared_job_params = self._prepare_run_job(job_execution)
        oozie_server = prepared_job_params['oozie_server']
        wf_dir = prepared_job_params['wf_dir']
        hdfs_user = prepared_job_params['hdfs_user']
        oozie_params = prepared_job_params['oozie_params']
        use_hbase_lib = prepared_job_params['use_hbase_lib']
        ctx = prepared_job_params['context']
        job_execution = prepared_job_params['job_execution']

        coord_configs = {"jobTracker": "${jobTracker}",
                         "nameNode": "${nameNode}"}

        coord_xml = self._create_coordinator_xml(coord_configs)

        self._upload_coordinator_file(oozie_server, wf_dir, coord_xml,
                                      hdfs_user)
        job_params = self._get_oozie_job_params(
            hdfs_user, None, oozie_params, use_hbase_lib,
            job_execution.job_configs.job_execution_info, wf_dir,
            "scheduled")

        client = self.get_client()
        oozie_job_id = client.add_job(x.create_hadoop_xml(job_params),
                                      job_execution)

        job_execution = conductor.job_execution_get(ctx, job_execution.id)

        if job_execution.info['status'] == edp.JOB_STATUS_TOBEKILLED:
            return (None, edp.JOB_STATUS_KILLED, None)
        try:
            status = client.get_job_status(job_execution,
                                           oozie_job_id)['status']
        except Exception:
            status = None
        return (oozie_job_id, status, None)

    @abc.abstractmethod
    def get_hdfs_user(self):
        pass

    @abc.abstractmethod
    def create_hdfs_dir(self, remote, dir_name):
        pass

    @abc.abstractmethod
    def get_oozie_server_uri(self, cluster):
        pass

    @abc.abstractmethod
    def get_oozie_server(self, cluster):
        pass

    @abc.abstractmethod
    def get_name_node_uri(self, cluster):
        pass

    @abc.abstractmethod
    def get_resource_manager_uri(self, cluster):
        pass

    def validate_job_execution(self, cluster, job, data):
        # Shell job type requires no specific fields
        if job.type == edp.JOB_TYPE_SHELL:
            return
        # All other types except Java require input and output
        # objects and Java require main class
        if job.type == edp.JOB_TYPE_JAVA:
            j.check_main_class_present(data, job)
        else:
            j.check_data_sources(data, job)

            job_type, subtype = edp.split_job_type(job.type)
            if job_type == edp.JOB_TYPE_MAPREDUCE and (
                    subtype == edp.JOB_SUBTYPE_STREAMING):
                j.check_streaming_present(data, job)

    @staticmethod
    def get_possible_job_config(job_type):
        return workflow_factory.get_possible_job_config(job_type)

    @staticmethod
    def get_supported_job_types():
        return [edp.JOB_TYPE_HIVE,
                edp.JOB_TYPE_JAVA,
                edp.JOB_TYPE_MAPREDUCE,
                edp.JOB_TYPE_MAPREDUCE_STREAMING,
                edp.JOB_TYPE_PIG,
                edp.JOB_TYPE_SHELL]

    def _prepare_job_binaries(self, job_binaries, r):
        for jb in job_binaries:
            jb_manager.JOB_BINARIES.get_job_binary_by_url(jb.url). \
                prepare_cluster(jb, remote=r)

    def _upload_job_files_to_hdfs(self, where, job_dir, job, configs,
                                  proxy_configs=None):

        mains = list(job.mains) if job.mains else []
        libs = list(job.libs) if job.libs else []
        builtin_libs = edp.get_builtin_binaries(job, configs)
        uploaded_paths = []
        hdfs_user = self.get_hdfs_user()
        job_dir_suffix = 'lib' if job.type != edp.JOB_TYPE_SHELL else ''
        lib_dir = os.path.join(job_dir, job_dir_suffix)

        with remote.get_remote(where) as r:
            job_binaries = mains + libs
            self._prepare_job_binaries(job_binaries, r)

            # upload mains
            uploaded_paths.extend(self._upload_job_binaries(r, mains,
                                                            proxy_configs,
                                                            hdfs_user,
                                                            job_dir))
            # upload libs
            if len(libs) and job_dir_suffix:
                # HDFS 2.2.0 fails to put file if the lib dir does not exist
                self.create_hdfs_dir(r, lib_dir)
            uploaded_paths.extend(self._upload_job_binaries(r, libs,
                                                            proxy_configs,
                                                            hdfs_user,
                                                            lib_dir))
            # upload buitin_libs
            for lib in builtin_libs:
                h.put_file_to_hdfs(r, lib['raw'], lib['name'], lib_dir,
                                   hdfs_user)
                uploaded_paths.append(lib_dir + lib['name'])
        return uploaded_paths

    def _upload_job_binaries(self, r, job_binaries, proxy_configs,
                             hdfs_user, job_dir):
        uploaded_paths = []
        for jb in job_binaries:
            path = jb_manager.JOB_BINARIES. \
                get_job_binary_by_url(jb.url). \
                copy_binary_to_cluster(jb, proxy_configs=proxy_configs,
                                       remote=r, context=context.ctx())

            h.copy_from_local(r, path, job_dir, hdfs_user)
            uploaded_paths.append(path)
        return uploaded_paths

    def _create_hdfs_workflow_dir(self, where, job):
        constructed_dir = '/user/%s/' % self.get_hdfs_user()
        constructed_dir = self._add_postfix(constructed_dir)
        constructed_dir += '%s/%s' % (job.name, uuidutils.generate_uuid())
        with remote.get_remote(where) as r:
            self.create_hdfs_dir(r, constructed_dir)

        return constructed_dir

    def _create_coordinator_xml(self, coord_configs, config_filter=None,
                                appname='coord'):
        doc = xml.Document()

        # Create the <coordinator-app> base element
        coord = doc.createElement('coordinator-app')
        coord.attributes['name'] = appname
        coord.attributes['start'] = "${start}"
        coord.attributes['end'] = "${end}"
        coord.attributes['frequency'] = "${frequency}"
        coord.attributes['timezone'] = 'UTC'
        coord.attributes['xmlns'] = 'uri:oozie:coordinator:0.2'

        doc.appendChild(coord)

        action = doc.createElement('action')
        workflow = doc.createElement('workflow')
        coord.appendChild(action)
        action.appendChild(workflow)
        x.add_text_element_to_tag(doc, "workflow", 'app-path',
                                  "${workflowAppUri}")
        configuration = doc.createElement('configuration')
        workflow.appendChild(configuration)

        default_configs = []
        if config_filter is not None:
            default_configs = [cfg['name'] for cfg in config_filter]

        for name in sorted(coord_configs):
            if name in default_configs or config_filter is None:
                x.add_property_to_configuration(doc, name, coord_configs[name])

        # Return newly created XML
        return doc.toprettyxml(indent="  ")

    def _add_postfix(self, constructed_dir):
        def _append_slash_if_needed(path):
            if path[-1] != '/':
                path += '/'
            return path

        constructed_dir = _append_slash_if_needed(constructed_dir)
        if CONF.job_workflow_postfix:
            constructed_dir = ''.join([str(constructed_dir),
                                       str(CONF.job_workflow_postfix)])
        return _append_slash_if_needed(constructed_dir)

    def _resolve_external_hdfs_urls(self, job_configs):
        external_hdfs_urls = []
        for k, v in six.iteritems(job_configs.get('configs', {})):
            if isinstance(v, six.string_types) and v.startswith("hdfs://"):
                external_hdfs_urls.append(v)
        for k, v in six.iteritems(job_configs.get('params', {})):
            if isinstance(v, six.string_types) and v.startswith("hdfs://"):
                external_hdfs_urls.append(v)
        for v in job_configs.get('args', []):
            if isinstance(v, six.string_types) and v.startswith("hdfs://"):
                external_hdfs_urls.append(v)
        return external_hdfs_urls

    def suspend_job(self, job_execution):
        return self._manage_job(job_execution, edp.JOB_ACTION_SUSPEND)

    def _manage_job(self, job_execution, action):
        if job_execution.oozie_job_id is not None:
            client = self.get_client()
            if action == edp.JOB_ACTION_SUSPEND:
                client.suspend_job(job_execution)
            return client.get_job_status(job_execution)



# Copyright 2016 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Add boot_from_volumes field for node_groups and related classes

Revision ID: 034
Revises: 033
Create Date: 2018-06-06 17:36:04.749264

"""

# revision identifiers, used by Alembic.
revision = '034'
down_revision = '033'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('node_group_templates',
                  sa.Column('boot_from_volume', sa.Boolean(), nullable=False))

    op.add_column('node_groups',
                  sa.Column('boot_from_volume', sa.Boolean(), nullable=False))

    op.add_column('templates_relations',
                  sa.Column('boot_from_volume', sa.Boolean(), nullable=False))



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module contains description of objects returned by the
conductor.

The actual objects returned are located in resource.py, which aim
is to hide some necessary magic. Current module describes objects
fields via docstrings and contains implementation of helper methods.
"""

import random

from oslo_config import cfg

from sahara.utils import configs
from sahara.utils import remote


CONF = cfg.CONF
CONF.import_opt('node_domain', 'sahara.config')


class Cluster(object):
    """An object representing Cluster.

    id
    name
    description
    tenant_id
    trust_id
    is_transient
    plugin_name
    hadoop_version
    cluster_configs - configs dict converted to object,
                      see the docs for details
    default_image_id
    anti_affinity
    anti_affinity_ratio
    management_private_key
    management_public_key
    user_keypair_id
    status
    status_description
    info
    extra
    rollback_info - internal information required for rollback
    sahara_info - internal information about sahara settings
    provision_progress - list of ProvisionStep objects
    node_groups - list of NodeGroup objects
    cluster_template_id
    cluster_template - ClusterTemplate object
    use_autoconfig
    is_public
    is_protected
    domain_name
    """

    def has_proxy_gateway(self):
        for ng in self.node_groups:
            if ng.is_proxy_gateway:
                return True

    def get_proxy_gateway_node(self):
        proxies = []
        for ng in self.node_groups:
            if ng.is_proxy_gateway and ng.instances:
                proxies += ng.instances

        if proxies:
            return random.choice(proxies)

        return None

    @property
    def stack_name(self):
        extra = self.extra or {}
        return extra.get('heat_stack_name', self.name)

    def use_designate_feature(self):
        return CONF.use_designate and self.domain_name


class NodeGroup(object):
    """An object representing Node Group.

    id
    name
    flavor_id
    image_id
    image_username
    node_processes - list of node processes
    node_configs - configs dict converted to object,
                   see the docs for details
    volumes_per_node
    volumes_size
    volumes_availability_zone - name of Cinder availability zone
                                where to spawn volumes
    volume_mount_prefix
    volume_type
    boot_from_volume - If set to True, the base image will be converted to a
                       bootable volume.
    boot_volume_type
    boot_volume_availability_zone - name of Cinder availability zone for
                                    spawning bootable volume.
    boot_volume_local_to_instance - indicates if boot volume and instance
                                    should be c reated on the same physical
                                    host.
    floating_ip_pool - Floating IP Pool name used to assign Floating IPs to
                       instances in this Node Group
    security_groups - List of security groups for instances in this Node Group
    auto_security_group - indicates if Sahara should create additional
                          security group for the Node Group
    availability_zone - name of Nova availability zone where to spawn instances
    open_ports - List of ports that will be opened if auto_security_group is
                 True
    is_proxy_gateway - indicates if nodes from this node group should be used
                       as proxy to access other cluster nodes
    volume_local_to_instance - indicates if volumes and instances should be
                               created on the same physical host

    count
    instances - list of Instance objects
    node_group_template_id
    node_group_template - NodeGroupTemplate object

    If node group belongs to cluster:
    cluster_id - parent Cluster ID
    cluster - parent Cluster object

    If node group belongs to cluster template:
    cluster_template_id - parent ClusterTemplate ID
    cluster_template - parent ClusterTemplate object
    """

    def configuration(self):
        return configs.merge_configs(self.cluster.cluster_configs,
                                     self.node_configs)

    def get_image_id(self):
        return self.image_id or self.cluster.default_image_id


class Instance(object):
    """An object representing Instance.

    id
    node_group_id - parent NodeGroup ID
    node_group - parent NodeGroup object
    instance_id - Nova instance ID
    instance_name
    internal_ip
    management_ip
    volumes
    storage_devices_number
    dns_hostname
    """

    def hostname(self):
        return self.instance_name

    def fqdn(self):
        if self._use_designate_feature():
            return self.dns_hostname
        else:
            return self.instance_name + '.' + CONF.node_domain

    def get_ip_or_dns_name(self):
        if self._use_designate_feature():
            return self.dns_hostname
        else:
            return self.management_ip

    def remote(self):
        return remote.get_remote(self)

    def storage_paths(self):
        mp = []
        for idx in range(1, self.storage_devices_number + 1):
            mp.append(self.node_group.volume_mount_prefix + str(idx))

        if not mp:
            mp = ['/mnt']

        return mp

    def _use_designate_feature(self):
        return CONF.use_designate and self.dns_hostname


class ClusterTemplate(object):
    """An object representing Cluster Template.

    id
    name
    description
    cluster_configs - configs dict converted to object,
                      see the docs for details
    default_image_id
    anti_affinity
    tenant_id
    plugin_name
    hadoop_version
    node_groups - list of NodeGroup objects
    is_public
    is_protected
    domain_name
    """


class NodeGroupTemplate(object):
    """An object representing Node Group Template.

    id
    name
    description
    tenant_id
    flavor_id
    image_id
    plugin_name
    hadoop_version
    node_processes - list of node processes
    node_configs - configs dict converted to object,
                   see the docs for details
    volumes_per_node
    volumes_size
    volumes_availability_zone
    volume_mount_prefix
    volume_type
    boot_from_volume
    boot_volume_type
    boot_volume_availability_zone
    boot_volume_local_to_instance
    floating_ip_pool
    security_groups
    auto_security_group
    availability_zone
    is_proxy_gateway
    volume_local_to_instance
    is_public
    is_protected
    """


class Image(object):
    """An object representing Image.

    id
    tags
    username
    description
    """


# EDP Objects

class DataSource(object):
    """An object representing Data Source.

    id
    tenant_id
    name
    description
    type
    url
    credentials
    is_public
    is_protected
    """


class JobExecution(object):
    """An object representing JobExecution

    id
    tenant_id
    job_id
    input_id
    output_id
    start_time
    end_time
    cluster_id
    info
    engine_job_id
    return_code
    job_configs
    interface
    extra
    data_source_urls
    is_public
    is_protected
    """


class Job(object):
    """An object representing Job

    id
    tenant_id
    name
    description
    type
    mains
    libs
    interface
    is_public
    is_protected
    """


class JobBinary(object):
    """An object representing JobBinary

    id
    tenant_id
    name
    description
    url -  URLs may be the following: internal-db://URL, swift://
    extra - extra may contain not only user-password but e.g. auth-token
    is_public
    is_protected
    """


class JobBinaryInternal(object):
    """An object representing JobBinaryInternal

    Note that the 'data' field is not returned. It uses deferred
    loading and must be requested explicitly with the
    job_binary_get_raw_data() conductor method.

    id
    tenant_id
    name
    datasize
    is_public
    is_protected
    """

# Events ops


class ClusterProvisionStep(object):
    """An object representing cluster ProvisionStep

    id
    cluster_id
    tenant_id
    step_name
    step_type
    total
    successful
    events - list of Events objects assigned to the cluster
    """


class ClusterEvent(object):
    """An object representing events about cluster provision

    id
    node_group_id
    instance_id
    instance_name
    event_info
    successful
    step_id
    """


class ClusterVerification(object):
    """An object representing cluster verification

    id
    cluster_id
    status
    checks
    """


class ClusterHealthCheck(object):
    """An object representing health check

    id
    verification_id
    status
    description
    name
    """



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_policy import policy

from sahara.common.policies import base


jobs_policies = [
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_JOBS % 'execute',
        check_str=base.UNPROTECTED,
        description='Run job.',
        operations=[{'path': '/v1.1/{project_id}/jobs/{job_id}/execute',
                     'method': 'POST'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_JOBS % 'get',
        check_str=base.UNPROTECTED,
        description='Show job details.',
        operations=[{'path': '/v1.1/{project_id}/jobs/{job_id}',
                     'method': 'GET'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_JOBS % 'create',
        check_str=base.UNPROTECTED,
        description='Create job.',
        operations=[{'path': '/v1.1/{project_id}/jobs', 'method': 'POST'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_JOBS % 'get_all',
        check_str=base.UNPROTECTED,
        description='List jobs.',
        operations=[{'path': '/v1.1/{project_id}/jobs', 'method': 'GET'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_JOBS % 'modify',
        check_str=base.UNPROTECTED,
        description='Update job object.',
        operations=[{'path': '/v1.1/{project_id}/jobs/{job_id}',
                     'method': 'PATCH'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_JOBS % 'get_config_hints',
        check_str=base.UNPROTECTED,
        description='Get job config hints.',
        operations=[
            {'path': '/v1.1/{project_id}/jobs/get_config_hints/{job_type}',
             'method': 'GET'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_JOBS % 'delete',
        check_str=base.UNPROTECTED,
        description='Remove job.',
        operations=[{'path': '/v1.1/{project_id}/jobs/{job_id}',
                     'method': 'DELETE'}]),
]


def list_rules():
    return jobs_policies



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from sahara.service.validations import node_group_template_schema as ngt_schema
from sahara.service.validations import shares


def _build_ng_schema_for_cluster_tmpl():
    cl_tmpl_ng_schema = copy.deepcopy(ngt_schema.NODE_GROUP_TEMPLATE_SCHEMA)
    cl_tmpl_ng_schema["properties"].update({"count": {"type": "integer"}})
    cl_tmpl_ng_schema["required"] = ["name", "flavor_id",
                                     "node_processes", "count"]

    del cl_tmpl_ng_schema["properties"]["plugin_name"]
    del cl_tmpl_ng_schema["properties"]["hadoop_version"]

    return cl_tmpl_ng_schema


_cluster_tmpl_ng_schema = _build_ng_schema_for_cluster_tmpl()


def _build_ng_tmpl_schema_for_cluster_template():
    cl_tmpl_ng_tmpl_schema = copy.deepcopy(_cluster_tmpl_ng_schema)
    cl_tmpl_ng_tmpl_schema["properties"].update(
        {
            "node_group_template_id": {
                "type": "string",
                "format": "uuid",
            }
        })
    cl_tmpl_ng_tmpl_schema["required"] = ["node_group_template_id",
                                          "name", "count"]
    return cl_tmpl_ng_tmpl_schema


_cluster_tmpl_ng_tmpl_schema = _build_ng_tmpl_schema_for_cluster_template()


CLUSTER_TEMPLATE_SCHEMA = {
    "type": "object",
    "properties": {
        "name": {
            "type": "string",
            "minLength": 1,
            "maxLength": 50,
            "format": "valid_name_hostname",
        },
        "plugin_name": {
            "type": "string",
        },
        "hadoop_version": {
            "type": "string",
        },
        "default_image_id": {
            "type": ["string", "null"],
            "format": "uuid",
        },
        "cluster_configs": {
            "type": ["configs", "null"],
        },
        "node_groups": {
            "type": ["array", "null"],
            "items": {
                "oneOf": [_cluster_tmpl_ng_tmpl_schema,
                          _cluster_tmpl_ng_schema]
            }
        },
        "anti_affinity": {
            "type": ["array", "null"],
            "items": {
                "type": "string",
            },
        },
        "description": {
            "type": ["string", "null"],
        },
        "neutron_management_network": {
            "type": ["string", "null"],
            "format": "uuid"
        },
        "shares": copy.deepcopy(shares.SHARE_SCHEMA),
        "use_autoconfig": {
            "type": ["boolean", "null"],
        },
        "is_public": {
            "type": ["boolean", "null"],
        },
        "is_protected": {
            "type": ["boolean", "null"],
        },
        "domain_name": {
            "type": ["string", "null"],
        }
    },
    "additionalProperties": False,
    "required": [
        "name",
        "plugin_name",
        "hadoop_version",
    ]
}

# APIv2: renaming hadoop_version -> plugin_version
CLUSTER_TEMPLATE_SCHEMA_V2 = copy.deepcopy(CLUSTER_TEMPLATE_SCHEMA)
del CLUSTER_TEMPLATE_SCHEMA_V2["properties"]["hadoop_version"]
CLUSTER_TEMPLATE_SCHEMA_V2["required"].remove("hadoop_version")
CLUSTER_TEMPLATE_SCHEMA_V2["properties"].update({
    "plugin_version": {
        "type": "string",
    }})
CLUSTER_TEMPLATE_SCHEMA_V2["required"].append("plugin_version")

CLUSTER_TEMPLATE_UPDATE_SCHEMA = copy.copy(CLUSTER_TEMPLATE_SCHEMA)
CLUSTER_TEMPLATE_UPDATE_SCHEMA["required"] = []

CLUSTER_TEMPLATE_UPDATE_SCHEMA_V2 = copy.copy(CLUSTER_TEMPLATE_SCHEMA_V2)
CLUSTER_TEMPLATE_UPDATE_SCHEMA_V2["required"] = []



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from sahara.service.edp.binary_retrievers import dispatch
from sahara.tests.unit import base


class TestDispatch(base.SaharaTestCase):
    def setUp(self):
        super(TestDispatch, self).setUp()

    @mock.patch('sahara.service.edp.s3_common.get_raw_job_binary_data')
    @mock.patch('sahara.service.edp.binary_retrievers.'
                'manila_share.get_file_info')
    @mock.patch(
        'sahara.service.edp.binary_retrievers.internal_swift.'
        'get_raw_data_with_context')
    @mock.patch(
        'sahara.service.edp.binary_retrievers.internal_swift.get_raw_data')
    @mock.patch('sahara.service.edp.binary_retrievers.sahara_db.get_raw_data')
    @mock.patch('sahara.context.ctx')
    def test_get_raw_binary(self, ctx, db_get_raw_data, i_s_get_raw_data,
                            i_s_get_raw_data_with_context, m_s_get_file_info,
                            s3_get_raw_jb_data):
        ctx.return_value = mock.Mock()

        job_binary = mock.Mock()
        job_binary.url = 'internal-db://somebinary'

        dispatch.get_raw_binary(job_binary)
        self.assertEqual(1, db_get_raw_data.call_count)

        job_binary.url = 'swift://container/object'
        proxy_configs = dict(proxy_username='proxytest',
                             proxy_password='proxysecret',
                             proxy_trust_id='proxytrust')
        dispatch.get_raw_binary(job_binary, proxy_configs)
        dispatch.get_raw_binary(job_binary, proxy_configs, with_context=True)
        dispatch.get_raw_binary(job_binary, with_context=True)
        self.assertEqual(1, i_s_get_raw_data.call_count)
        self.assertEqual(2, i_s_get_raw_data_with_context.call_count)

        job_binary.url = 'manila://the_share_id/the_path'
        remote = mock.Mock()
        remote.instance.node_group.cluster.shares = []
        remote.instance.node_group.shares = []
        dispatch.get_raw_binary(job_binary, remote=remote)
        self.assertEqual(1, m_s_get_file_info.call_count)

        job_binary.url = 's3://bucket/object.jar'
        dispatch.get_raw_binary(job_binary)
        self.assertEqual(1, s3_get_raw_jb_data.call_count)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import traceback

import eventlet
from eventlet.green import threading
from eventlet.green import time
from eventlet import greenpool
from eventlet import semaphore
from oslo_config import cfg
from oslo_context import context
from oslo_log import log as logging

from sahara import exceptions as ex
from sahara.i18n import _
from sahara.service import sessions


CONF = cfg.CONF
LOG = logging.getLogger(__name__)


class Context(context.RequestContext):
    def __init__(self,
                 user_id=None,
                 tenant_id=None,
                 auth_token=None,
                 service_catalog=None,
                 username=None,
                 tenant_name=None,
                 roles=None,
                 is_admin=None,
                 remote_semaphore=None,
                 resource_uuid=None,
                 current_instance_info=None,
                 request_id=None,
                 auth_plugin=None,
                 overwrite=True,
                 **kwargs):
        if kwargs:
            LOG.warning('Arguments dropped when creating context: '
                        '{args}'.format(args=kwargs))

        super(Context, self).__init__(auth_token=auth_token,
                                      user=user_id,
                                      tenant=tenant_id,
                                      is_admin=is_admin,
                                      resource_uuid=resource_uuid,
                                      request_id=request_id,
                                      roles=roles)
        self.service_catalog = service_catalog
        self.username = username
        self.tenant_name = tenant_name
        self.remote_semaphore = remote_semaphore or semaphore.Semaphore(
            CONF.cluster_remote_threshold)
        self.auth_plugin = auth_plugin
        if overwrite or not hasattr(context._request_store, 'context'):
            self.update_store()

        if current_instance_info is not None:
            self.current_instance_info = current_instance_info
        else:
            self.current_instance_info = InstanceInfo()

    def clone(self):
        return Context(
            self.user_id,
            self.tenant_id,
            self.auth_token,
            self.service_catalog,
            self.username,
            self.tenant_name,
            self.roles,
            self.is_admin,
            self.remote_semaphore,
            self.resource_uuid,
            self.current_instance_info,
            self.request_id,
            self.auth_plugin,
            overwrite=False)

    def to_dict(self):
        d = super(Context, self).to_dict()
        d.update({
            'user_id': self.user_id,
            'tenant_id': self.tenant_id,
            'service_catalog': self.service_catalog,
            'username': self.username,
            'tenant_name': self.tenant_name,
            'user_name': self.username,
            'project_name': self.tenant_name})
        return d

    def is_auth_capable(self):
        return (self.service_catalog and self.auth_token and self.tenant and
                self.user_id)

    # NOTE(adrienverge): The Context class uses the 'user' and 'tenant'
    # properties internally (inherited from oslo_context), but Sahara code
    # often uses 'user_id' and 'tenant_id'.
    @property
    def user_id(self):
        return self.user

    @user_id.setter
    def user_id(self, value):
        self.user = value

    @property
    def tenant_id(self):
        return self.tenant

    @tenant_id.setter
    def tenant_id(self, value):
        self.tenant = value


def get_admin_context():
    return Context(is_admin=True, overwrite=False)


_CTX_STORE = threading.local()
_CTX_KEY = 'current_ctx'


def has_ctx():
    return hasattr(_CTX_STORE, _CTX_KEY)


def ctx():
    if not has_ctx():
        raise ex.IncorrectStateError(_("Context isn't available here"))
    return getattr(_CTX_STORE, _CTX_KEY)


def current():
    return ctx()


def set_ctx(new_ctx):
    if not new_ctx and has_ctx():
        delattr(_CTX_STORE, _CTX_KEY)
        if hasattr(context._request_store, 'context'):
            delattr(context._request_store, 'context')

    if new_ctx:
        setattr(_CTX_STORE, _CTX_KEY, new_ctx)
        setattr(context._request_store, 'context', new_ctx)


def _wrapper(ctx, thread_description, thread_group, func, *args, **kwargs):
    try:
        set_ctx(ctx)
        func(*args, **kwargs)
    except BaseException as e:
        LOG.debug(
            "Thread {thread} failed with exception: {exception}".format(
                thread=thread_description, exception=e))
        if thread_group and not thread_group.exc:
            thread_group.exc = e
            thread_group.exc_stacktrace = traceback.format_exc()
            thread_group.failed_thread = thread_description
    finally:
        if thread_group:
            thread_group._on_thread_exit()

        set_ctx(None)


def spawn(thread_description, func, *args, **kwargs):
    eventlet.spawn(_wrapper, current().clone(), thread_description,
                   None, func, *args, **kwargs)


class ThreadGroup(object):
    """ThreadGroup object.

    It is advised to use TreadGroup as a context manager instead
    of instantiating and calling _wait() manually. The __exit__()
    guaranties to exit only after all child threads are done, even if
    spawning code have thrown an exception
    """

    def __init__(self, thread_pool_size=1000):
        self.tg = greenpool.GreenPool(size=thread_pool_size)
        self.exc = None
        self.exc_stacktrace = None
        self.failed_thread = None
        self.threads = 0
        self.cv = threading.Condition()

    def spawn(self, thread_description, func, *args, **kwargs):
        self.tg.spawn(_wrapper, current().clone(), thread_description,
                      self, func, *args, **kwargs)

        with self.cv:
            self.threads += 1

    def _on_thread_exit(self):
        with self.cv:
            self.threads -= 1
            if self.threads == 0:
                self.cv.notifyAll()

    # NOTE(dmitryme): A little rationale on why we reimplemented wait():
    # * Eventlet's GreenPool.wait() can hung
    # * Oslo's ThreadGroup.wait() can exit before all threads are done
    #
    def _wait(self):
        """Using of _wait() method.

        It is preferred to use the class as a context manager and do not
        use _wait() directly, see class docstring for an explanation.
        """
        with self.cv:
            while self.threads > 0:
                self.cv.wait()

        if self.exc:
            raise ex.ThreadException(self.failed_thread, self.exc,
                                     self.exc_stacktrace)

    def __enter__(self):
        return self

    def __exit__(self, *ex):
        if not any(ex):
            self._wait()
        else:
            # If spawning code thrown an exception, it had higher priority
            # for us than the one thrown inside child thread (if any)
            try:
                self._wait()
            except Exception:
                # that will make __exit__ throw original exception
                pass


def sleep(seconds=0):
    time.sleep(seconds)


class InstanceInfo(object):
    def __init__(self, cluster_id=None, instance_id=None, instance_name=None,
                 node_group_id=None, step_type=None, step_id=None):
        self.cluster_id = cluster_id
        self.instance_id = instance_id
        self.instance_name = instance_name
        self.node_group_id = node_group_id
        self.step_type = step_type
        self.step_id = step_id


def set_step_type(step_type):
    current().current_instance_info.step_type = step_type


class InstanceInfoManager(object):
    def __init__(self, instance_info):
        self.prev_instance_info = current().current_instance_info
        if not instance_info.step_type:
            instance_info.step_type = self.prev_instance_info.step_type
        if not instance_info.step_id:
            instance_info.step_id = self.prev_instance_info.step_id
        current().current_instance_info = instance_info

    def __enter__(self):
        pass

    def __exit__(self, *args):
        current().current_instance_info = self.prev_instance_info


def set_current_cluster_id(cluster_id):
    current().resource_uuid = 'none, cluster: %s' % cluster_id


def set_current_job_execution_id(je_id):
    current().resource_uuid = 'none, job_execution: %s' % je_id


class SetCurrentInstanceId(object):
    def __init__(self, instance_id):
        ctx = current()
        self.prev_uuid = ctx.resource_uuid
        if ctx.resource_uuid:
            ctx.resource_uuid = ctx.resource_uuid.replace('none', instance_id)
            context.get_current().resource_uuid = ctx.resource_uuid

    def __enter__(self):
        pass

    def __exit__(self, *ex):
        current().resource_uuid = self.prev_uuid
        context.get_current().resource_uuid = self.prev_uuid


def set_current_instance_id(instance_id):
    return SetCurrentInstanceId(instance_id)


def get_auth_token():
    cur = current()
    if cur.auth_plugin:
        try:
            cur.auth_token = sessions.cache().token_for_auth(cur.auth_plugin)
        except Exception as e:
            LOG.warning("Cannot update token, reason: %s", e)
    return cur.auth_token



#    Copyright 2015 EasyStack Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from sahara.utils.hacking import checks


class HackingTestCase(testtools.TestCase):
    def test_dict_constructor_with_list_copy(self):
        # Following checks for code-lines with pep8 error
        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "    dict([(i, connect_info[i])"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "    attrs = dict([(k, _from_json(v))"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "        type_names = dict((value, key) for key, value in"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "   dict((value, key) for key, value in"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "foo(param=dict((k, v) for k, v in bar.items()))"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            " dict([[i,i] for i in range(3)])"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "  dd = dict([i,i] for i in range(3))"))))
        # Following checks for ok code-lines
        self.assertEqual(0, len(list(checks.dict_constructor_with_list_copy(
            " dict()"))))

        self.assertEqual(0, len(list(checks.dict_constructor_with_list_copy(
            "        create_kwargs = dict(snapshot=snapshot,"))))

        self.assertEqual(0, len(list(checks.dict_constructor_with_list_copy(
            "      self._render_dict(xml, data_el, data.__dict__)"))))

    def test_use_jsonutils(self):
        self.assertEqual(0, len(list(checks.use_jsonutils(
            "import json    # noqa", "path"))))
        self.assertEqual(0, len(list(checks.use_jsonutils(
            "from oslo_serialization import jsonutils as json", "path"))))
        self.assertEqual(0, len(list(checks.use_jsonutils(
            "import jsonschema", "path"))))
        self.assertEqual(1, len(list(checks.use_jsonutils(
            "import json", "path"))))
        self.assertEqual(1, len(list(checks.use_jsonutils(
            "import json as jsonutils", "path"))))

    def test_no_mutable_default_args(self):
        self.assertEqual(0, len(list(checks.no_mutable_default_args(
            "def foo (bar):"))))
        self.assertEqual(1, len(list(checks.no_mutable_default_args(
            "def foo (bar=[]):"))))
        self.assertEqual(1, len(list(checks.no_mutable_default_args(
            "def foo (bar={}):"))))



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Handles database requests from other Sahara services."""

import copy

from oslo_config import cfg

from sahara.conductor import resource as r
from sahara.db import base as db_base
from sahara.service.castellan import utils as key_manager
from sahara.service.edp.utils import shares
from sahara.utils import configs
from sahara.utils import crypto


CONF = cfg.CONF

CLUSTER_DEFAULTS = {
    "cluster_configs": {},
    "status": "undefined",
    "anti_affinity": [],
    "anti_affinity_ratio": 1,
    "status_description": "",
    "info": {},
    "rollback_info": {},
    "sahara_info": {},
    "is_public": False,
    "is_protected": False
}

NODE_GROUP_DEFAULTS = {
    "node_processes": [],
    "node_configs": {},
    "volumes_per_node": 0,
    "volumes_size": 0,
    "volumes_availability_zone": None,
    "volume_mount_prefix": "/volumes/disk",
    "volume_type": None,
    "boot_from_volume": False,
    "boot_volume_type": None,
    "boot_volume_availability_zone": None,
    "boot_volume_local_to_instance": False,
    "floating_ip_pool": None,
    "security_groups": None,
    "auto_security_group": False,
    "availability_zone": None,
    "is_proxy_gateway": False,
    "volume_local_to_instance": False,
}

NODE_GROUP_TEMPLATE_DEFAULTS = copy.deepcopy(NODE_GROUP_DEFAULTS)
NODE_GROUP_TEMPLATE_DEFAULTS.update({"is_public": False,
                                     "is_protected": False})

INSTANCE_DEFAULTS = {
    "volumes": [],
    "storage_devices_number": 0
}

DATA_SOURCE_DEFAULTS = {
    "credentials": {},
    "is_public": False,
    "is_protected": False
}

JOB_DEFAULTS = {
    "is_public": False,
    "is_protected": False
}

JOB_BINARY_DEFAULTS = {
    "is_public": False,
    "is_protected": False
}

JOB_BINARY_INTERNAL_DEFAULTS = {
    "is_public": False,
    "is_protected": False
}

JOB_EXECUTION_DEFAULTS = {
    "is_public": False,
    "is_protected": False
}


def _apply_defaults(values, defaults):
    new_values = copy.deepcopy(defaults)
    new_values.update(values)
    return new_values


class ConductorManager(db_base.Base):
    """This class aimed to conduct things.

    The methods in the base API for sahara-conductor are various proxy
    operations that allow other services to get specific work done without
    locally accessing the database.

    Additionally it performs some template-to-object copying magic.
    """

    def __init__(self):
        super(ConductorManager, self).__init__()

    # Common helpers

    def _populate_node_groups(self, context, cluster):
        node_groups = cluster.get('node_groups')
        if not node_groups:
            return []

        populated_node_groups = []
        for node_group in node_groups:
            populated_node_group = self._populate_node_group(context,
                                                             node_group)
            self._cleanup_node_group(populated_node_group)
            populated_node_group["tenant_id"] = context.tenant_id
            populated_node_groups.append(
                populated_node_group)

        return populated_node_groups

    def _cleanup_node_group(self, node_group):
        node_group.pop('id', None)
        node_group.pop('created_at', None)
        node_group.pop('updated_at', None)

    def _populate_node_group(self, context, node_group):
        node_group_merged = copy.deepcopy(NODE_GROUP_DEFAULTS)

        ng_tmpl_id = node_group.get('node_group_template_id')
        ng_tmpl = None
        if ng_tmpl_id:
            ng_tmpl = self.node_group_template_get(context, ng_tmpl_id)

            self._cleanup_node_group(ng_tmpl)
            node_group_merged.update(ng_tmpl)

        node_group_merged.update(node_group)

        if ng_tmpl:
            node_group_merged['node_configs'] = configs.merge_configs(
                ng_tmpl.get('node_configs'),
                node_group.get('node_configs'))

        return node_group_merged

    # Cluster ops

    def cluster_get(self, context, cluster, show_progress=False):
        """Return the cluster or None if it does not exist."""
        return self.db.cluster_get(context, cluster, show_progress)

    def cluster_get_all(self, context, regex_search=False, **kwargs):
        """Get all clusters filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self.db.cluster_get_all(context, regex_search, **kwargs)

    def cluster_create(self, context, values):
        """Create a cluster from the values dictionary."""

        # loading defaults
        merged_values = copy.deepcopy(CLUSTER_DEFAULTS)
        merged_values['tenant_id'] = context.tenant_id

        private_key, public_key = crypto.generate_key_pair()
        merged_values['management_private_key'] = private_key
        merged_values['management_public_key'] = public_key

        cluster_template_id = values.get('cluster_template_id')
        c_tmpl = None

        if cluster_template_id:
            c_tmpl = self.cluster_template_get(context, cluster_template_id)

            del c_tmpl['created_at']
            del c_tmpl['updated_at']
            del c_tmpl['id']
            del c_tmpl['is_public']
            del c_tmpl['is_protected']
            del c_tmpl['tenant_id']

            # updating with cluster_template values
            merged_values.update(c_tmpl)

        # updating with values provided in request
        merged_values.update(values)

        if c_tmpl:
            merged_values['cluster_configs'] = configs.merge_configs(
                c_tmpl.get('cluster_configs'),
                values.get('cluster_configs'))

        merged_values['node_groups'] = self._populate_node_groups(
            context, merged_values)

        return self.db.cluster_create(context, merged_values)

    def cluster_update(self, context, cluster, values):
        """Set the given properties on cluster and update it."""
        values = copy.deepcopy(values)
        update_shares = values.get('shares')
        if update_shares:
            original_shares = (
                self.db.cluster_get(context, cluster).get('shares', []))

        updated_cluster = self.db.cluster_update(context, cluster, values)
        if update_shares:
            for share in update_shares:
                # Only call mount_shares if we have new shares to mount.
                # We only need one positive case to bother calling mount_shares
                if share not in original_shares:
                    shares.mount_shares(r.ClusterResource(updated_cluster))
                    break
            # Any shares that were on the original, but not on the updated
            # list will be unmounted
            unmount_list = [share for share in original_shares
                            if share not in update_shares]
            if len(unmount_list) > 0:
                shares.unmount_shares(r.ClusterResource(updated_cluster),
                                      unmount_list)

        return updated_cluster

    def cluster_destroy(self, context, cluster):
        """Destroy the cluster or raise if it does not exist."""
        self.db.cluster_destroy(context, cluster)

    # Node Group ops

    def node_group_add(self, context, cluster, values):
        """Create a Node Group from the values dictionary."""
        values = copy.deepcopy(values)
        values = self._populate_node_group(context, values)
        values['tenant_id'] = context.tenant_id
        return self.db.node_group_add(context, cluster, values)

    def node_group_update(self, context, node_group, values):
        """Set the given properties on node_group and update it."""
        values = copy.deepcopy(values)
        self.db.node_group_update(context, node_group, values)

    def node_group_remove(self, context, node_group):
        """Destroy the node_group or raise if it does not exist."""
        self.db.node_group_remove(context, node_group)

    # Instance ops

    def instance_add(self, context, node_group, values):
        """Create an Instance from the values dictionary."""
        values = copy.deepcopy(values)
        values = _apply_defaults(values, INSTANCE_DEFAULTS)
        values['tenant_id'] = context.tenant_id
        return self.db.instance_add(context, node_group, values)

    def instance_update(self, context, instance, values):
        """Set the given properties on Instance and update it."""
        values = copy.deepcopy(values)
        self.db.instance_update(context, instance, values)

    def instance_remove(self, context, instance):
        """Destroy the Instance or raise if it does not exist."""
        self.db.instance_remove(context, instance)

    # Volumes ops

    def append_volume(self, context, instance, volume_id):
        """Append volume_id to instance."""
        self.db.append_volume(context, instance, volume_id)

    def remove_volume(self, context, instance, volume_id):
        """Remove volume_id in instance."""
        self.db.remove_volume(context, instance, volume_id)

    # Cluster Template ops

    def cluster_template_get(self, context, cluster_template):
        """Return the cluster_template or None if it does not exist."""
        return self.db.cluster_template_get(context, cluster_template)

    def cluster_template_get_all(self, context, regex_search=False, **kwargs):
        """Get all cluster templates filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self.db.cluster_template_get_all(context,
                                                regex_search, **kwargs)

    def cluster_template_create(self, context, values):
        """Create a cluster_template from the values dictionary."""
        values = copy.deepcopy(values)
        values = _apply_defaults(values, CLUSTER_DEFAULTS)
        values['tenant_id'] = context.tenant_id

        values['node_groups'] = self._populate_node_groups(context, values)

        return self.db.cluster_template_create(context, values)

    def cluster_template_destroy(self, context, cluster_template,
                                 ignore_prot_on_def=False):
        """Destroy the cluster_template or raise if it does not exist."""
        self.db.cluster_template_destroy(context, cluster_template,
                                         ignore_prot_on_def)

    def cluster_template_update(self, context, id, values,
                                ignore_prot_on_def=False):
        """Update a cluster_template from the values dictionary."""
        values = copy.deepcopy(values)
        values['tenant_id'] = context.tenant_id
        values['id'] = id

        if 'node_groups' in values:
            values['node_groups'] = self._populate_node_groups(context, values)

        return self.db.cluster_template_update(context, values,
                                               ignore_prot_on_def)

    # Node Group Template ops

    def node_group_template_get(self, context, node_group_template):
        """Return the Node Group Template or None if it does not exist."""
        return self.db.node_group_template_get(context, node_group_template)

    def node_group_template_get_all(self,
                                    context, regex_search=False, **kwargs):
        """Get all NodeGroupTemplates filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self.db.node_group_template_get_all(context,
                                                   regex_search, **kwargs)

    def node_group_template_create(self, context, values):
        """Create a Node Group Template from the values dictionary."""
        values = copy.deepcopy(values)
        values = _apply_defaults(values, NODE_GROUP_TEMPLATE_DEFAULTS)
        values['tenant_id'] = context.tenant_id

        return self.db.node_group_template_create(context, values)

    def node_group_template_destroy(self, context, node_group_template,
                                    ignore_prot_on_def=False):
        """Destroy the Node Group Template or raise if it does not exist."""
        self.db.node_group_template_destroy(context, node_group_template,
                                            ignore_prot_on_def)

    def node_group_template_update(self, context, id, values,
                                   ignore_prot_on_def=False):
        """Update a Node Group Template from the values dictionary."""
        values = copy.deepcopy(values)
        values['tenant_id'] = context.tenant_id
        values['id'] = id

        return self.db.node_group_template_update(context, values,
                                                  ignore_prot_on_def)

    # Data Source ops

    def data_source_get(self, context, data_source):
        """Return the Data Source or None if it does not exist."""
        return self.db.data_source_get(context, data_source)

    def data_source_get_all(self, context, regex_search=False, **kwargs):
        """Get all Data Sources filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self.db.data_source_get_all(context, regex_search, **kwargs)

    def data_source_count(self, context, **kwargs):
        """Count Data Sources filtered by **kwargs.

        Uses sqlalchemy "in_" clause for any tuple values
        Uses sqlalchemy "like" clause for any string values containing %
        """
        return self.db.data_source_count(context, **kwargs)

    def data_source_create(self, context, values):
        """Create a Data Source from the values dictionary."""

        values = copy.deepcopy(values)
        values = _apply_defaults(values, DATA_SOURCE_DEFAULTS)
        values['tenant_id'] = context.tenant_id
        # if credentials are being passed in, we use the key_manager
        # to store the password.
        if (values.get('credentials') and
                values['credentials'].get('password')):
            values['credentials']['password'] = key_manager.store_secret(
                values['credentials']['password'], context)
        if (values.get('credentials') and
                values['credentials'].get('secretkey')):
            values['credentials']['secretkey'] = key_manager.store_secret(
                values['credentials']['secretkey'], context)
        return self.db.data_source_create(context, values)

    def data_source_destroy(self, context, data_source):
        """Destroy the Data Source or raise if it does not exist."""

        # in cases where the credentials to access the data source are
        # stored with the record and the external key manager is being
        # used, we need to delete the key from the external manager.
        if (CONF.use_barbican_key_manager and not
                CONF.use_domain_for_proxy_users):
            ds_record = self.data_source_get(context, data_source)
            if (ds_record.get('credentials') and
                    ds_record['credentials'].get('password')):
                key_manager.delete_secret(
                    ds_record['credentials']['password'], context)
        if CONF.use_barbican_key_manager:
            if (ds_record.get('credentials') and
                    ds_record['credentials'].get('secretkey')):
                key_manager.delete_secret(
                    ds_record['credentials']['secretkey'], context)
        return self.db.data_source_destroy(context, data_source)

    def data_source_update(self, context, id, values):
        """Update the Data Source or raise if it does not exist."""

        values = copy.deepcopy(values)
        values["id"] = id
        # in cases where the credentials to access the data source are
        # stored with the record and the external key manager is being
        # used, we need to delete the old key from the manager and
        # create a new one. the other option here would be to retrieve
        # the previous key and check to see if it has changed, but it
        # seems less expensive to just delete the old and create a new
        # one.
        # it should be noted that the jsonschema validation ensures that
        # if the proxy domain is not in use then credentials must be
        # sent with this record.

        # first we retrieve the original record to get the old key
        # uuid, and delete it.
        # next we create the new key.

        if CONF.use_barbican_key_manager:
            ds_record = self.data_source_get(context, id)
            if (ds_record.get('credentials') and
                    ds_record['credentials'].get('password') and
                    not CONF.use_domain_for_proxy_users):
                key_manager.delete_secret(
                    ds_record['credentials']['password'], context)
            if (values.get('credentials') and
                    values['credentials'].get('password') and
                    not CONF.use_domain_for_proxy_users):
                values['credentials']['password'] = key_manager.store_secret(
                    values['credentials']['password'], context)
            if (ds_record.get('credentials') and
                    ds_record['credentials'].get('secretkey')):
                key_manager.delete_secret(
                    ds_record['credentials']['secretkey'], context)
            if (values.get('credentials') and
                    values['credentials'].get('secretkey')):
                values['credentials']['secretkey'] = key_manager.store_secret(
                    values['credentials']['secretkey'], context)
        return self.db.data_source_update(context, values)

    # JobExecution ops

    def job_execution_get(self, context, job_execution):
        """Return the JobExecution or None if it does not exist."""
        return self.db.job_execution_get(context, job_execution)

    def job_execution_get_all(self, context, regex_search=False, **kwargs):
        """Get all JobExecutions filtered by **kwargs.

        kwargs key values may be the names of fields in a JobExecution
        plus the following special values with the indicated meaning:

        'cluster.name' -- name of the Cluster referenced by the JobExecution
        'job.name' -- name of the Job referenced by the JobExecution
        'status' -- JobExecution['info']['status']

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self.db.job_execution_get_all(context, regex_search, **kwargs)

    def job_execution_count(self, context, **kwargs):
        """Count number of JobExecutions filtered by **kwargs.

        e.g. job_execution_count(cluster_id=12, input_id=123)
        """
        return self.db.job_execution_count(context, **kwargs)

    def job_execution_create(self, context, values):
        """Create a JobExecution from the values dictionary."""
        values = copy.deepcopy(values)
        values = _apply_defaults(values, JOB_EXECUTION_DEFAULTS)
        values['tenant_id'] = context.tenant_id
        return self.db.job_execution_create(context, values)

    def job_execution_update(self, context, job_execution, values):
        """Updates a JobExecution from the values dictionary."""
        values = copy.deepcopy(values)
        return self.db.job_execution_update(context, job_execution, values)

    def job_execution_destroy(self, context, job_execution):
        """Destroy the JobExecution or raise if it does not exist."""
        return self.db.job_execution_destroy(context, job_execution)

    # Job ops

    def job_get(self, context, job):
        """Return the Job or None if it does not exist."""
        return self.db.job_get(context, job)

    def job_get_all(self, context, regex_search=False, **kwargs):
        """Get all Jobs filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self.db.job_get_all(context, regex_search, **kwargs)

    def job_create(self, context, values):
        """Create a Job from the values dictionary."""
        values = copy.deepcopy(values)
        values = _apply_defaults(values, JOB_DEFAULTS)
        values['tenant_id'] = context.tenant_id
        return self.db.job_create(context, values)

    def job_update(self, context, job, values):
        """Updates a Job from the values dictionary."""
        return self.db.job_update(context, job, values)

    def job_destroy(self, context, job):
        """Destroy the Job or raise if it does not exist."""
        self.db.job_destroy(context, job)

    # JobBinary ops

    def job_binary_get_all(self, context, regex_search=False, **kwargs):
        """Get all JobBinarys filtered by **kwargs.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search

        e.g.  job_binary_get_all(name='wordcount.jar')
        """
        return self.db.job_binary_get_all(context,
                                          regex_search, **kwargs)

    def job_binary_get(self, context, job_binary_id):
        """Return the JobBinary or None if it does not exist."""
        return self.db.job_binary_get(context, job_binary_id)

    def job_binary_create(self, context, values):
        """Create a JobBinary from the values dictionary."""

        values = copy.deepcopy(values)
        values = _apply_defaults(values, JOB_BINARY_DEFAULTS)
        values['tenant_id'] = context.tenant_id
        # if credentials are being passed in, we use the key_manager
        # to store the password.
        if values.get('extra') and values['extra'].get('password'):
            values['extra']['password'] = key_manager.store_secret(
                values['extra']['password'], context)
        if values.get('extra') and values['extra'].get('secretkey'):
            values['extra']['secretkey'] = key_manager.store_secret(
                values['extra']['secretkey'], context)
        return self.db.job_binary_create(context, values)

    def job_binary_destroy(self, context, job_binary):
        """Destroy the JobBinary or raise if it does not exist."""

        # in cases where the credentials to access the job binary are
        # stored with the record and the external key manager is being
        # used, we need to delete the key from the external manager.
        if CONF.use_barbican_key_manager:
            jb_record = self.job_binary_get(context, job_binary)
            if not CONF.use_domain_for_proxy_users:
                if (jb_record.get('extra') and
                        jb_record['extra'].get('password')):
                    key_manager.delete_secret(jb_record['extra']['password'],
                                              context)
            if (jb_record.get('extra') and
                    jb_record['extra'].get('secretkey')):
                key_manager.delete_secret(jb_record['extra']['secretkey'],
                                          context)
        self.db.job_binary_destroy(context, job_binary)

    def job_binary_update(self, context, id, values):
        """Update a JobBinary from the values dictionary."""

        values = copy.deepcopy(values)
        values['id'] = id
        # in cases where the credentials to access the job binary are
        # stored with the record and the external key manager is being
        # used, we need to delete the old key from the manager and
        # create a new one. the other option here would be to retrieve
        # the previous key and check to see if it has changed, but it
        # seems less expensive to just delete the old and create a new
        # one.
        if CONF.use_barbican_key_manager:
            # first we retrieve the original record to get the old key
            # uuid, and delete it.
            # next we create the new key.
            jb_record = self.job_binary_get(context, id)
            if not CONF.use_domain_for_proxy_users:
                if (jb_record.get('extra') and
                        jb_record['extra'].get('password')):
                    key_manager.delete_secret(jb_record['extra']['password'],
                                              context)
                if values.get('extra') and values['extra'].get('password'):
                    values['extra']['password'] = key_manager.store_secret(
                        values['extra']['password'], context)
            if jb_record.get('extra') and jb_record['extra'].get('secretkey'):
                key_manager.delete_secret(jb_record['extra']['secretkey'],
                                          context)
            if values.get('extra') and values['extra'].get('secretkey'):
                values['extra']['secretkey'] = key_manager.store_secret(
                    values['extra']['secretkey'], context)
        return self.db.job_binary_update(context, values)

    # JobBinaryInternal ops

    def job_binary_internal_get_all(self, context,
                                    regex_search=False, **kwargs):
        """Get all JobBinaryInternals filtered by **kwargs.

        The JobBinaryInternals returned do not contain a data field.

        :param context: The context, and associated authentication, to use with
                        this operation

        :param regex_search: If True, enable regex matching for filter
                             values. See the user guide for more information
                             on how regex matching is handled. If False,
                             no regex matching is done.

        :param kwargs: Specifies values for named fields by which
                       to constrain the search
        """
        return self.db.job_binary_internal_get_all(context,
                                                   regex_search, **kwargs)

    def job_binary_internal_get(self, context, job_binary_internal_id):
        """Return the JobBinaryInternal or None if it does not exist

        The JobBinaryInternal returned does not contain a data field.
        """
        return self.db.job_binary_internal_get(context, job_binary_internal_id)

    def job_binary_internal_create(self, context, values):
        """Create a JobBinaryInternal from the values dictionary."""

        # Since values["data"] is (should be) encoded as a string
        # here the deepcopy of values only incs a reference count on data.
        # This is nice, since data could be big...
        values = copy.deepcopy(values)
        values = _apply_defaults(values, JOB_BINARY_INTERNAL_DEFAULTS)
        values['tenant_id'] = context.tenant_id
        return self.db.job_binary_internal_create(context, values)

    def job_binary_internal_destroy(self, context, job_binary_internal):
        """Destroy the JobBinaryInternal or raise if it does not exist."""
        self.db.job_binary_internal_destroy(context, job_binary_internal)

    def job_binary_internal_get_raw_data(self,
                                         context, job_binary_internal_id):
        """Return the binary data field from a JobBinaryInternal."""
        return self.db.job_binary_internal_get_raw_data(
            context,
            job_binary_internal_id)

    def job_binary_internal_update(self, context, id, values):
        """Updates a JobBinaryInternal from the values dictionary."""
        return self.db.job_binary_internal_update(context, id, values)

    # Events ops

    def cluster_provision_step_add(self, context, cluster_id, values):
        """Create a provisioning step assigned to cluster from values dict."""
        return self.db.cluster_provision_step_add(context, cluster_id, values)

    def cluster_provision_step_update(self, context, provision_step):
        """Update the cluster provisioning step."""
        return self.db.cluster_provision_step_update(context, provision_step)

    def cluster_provision_progress_update(self, context, cluster_id):
        """Return cluster with provision progress updated field."""
        return self.db.cluster_provision_progress_update(context, cluster_id)

    def cluster_event_add(self, context, provision_step, values):
        """Assign new event to the specified provision step."""
        return self.db.cluster_event_add(context, provision_step, values)

    # Cluster verifications / health checks ops

    def cluster_verification_add(self, context, cluster_id, values):
        """Return created verification for the specified cluster."""
        return self.db.cluster_verification_add(context, cluster_id, values)

    def cluster_verification_get(self, context, verification_id):
        """Return verification with the specified verification_id."""
        return self.db.cluster_verification_get(context, verification_id)

    def cluster_verification_update(self, context, verification_id, values):
        """Return updated verification with the specified verification_id."""
        return self.db.cluster_verification_update(
            context, verification_id, values)

    def cluster_verification_delete(self, context, verification_id):
        """"Delete verification with the specified id."""
        return self.db.cluster_verification_delete(context, verification_id)

    def cluster_health_check_add(self, context, verification_id, values):
        """Return created health check in the specified verification."""
        return self.db.cluster_health_check_add(
            context, verification_id, values)

    def cluster_health_check_get(self, context, health_check_id):
        """Return health check with the specified health_check_id."""
        return self.db.cluster_health_check_get(context, health_check_id)

    def cluster_health_check_update(self, context, health_check_id, values):
        """Return updated health check with the specified health_check_id."""
        return self.db.cluster_health_check_update(
            context, health_check_id, values)

    def plugin_create(self, context, values):
        """Return created DB entry for plugin."""
        return self.db.plugin_create(context, values)

    def plugin_get(self, context, name):
        """Return DB entry for plugin."""
        return self.db.plugin_get(context, name)

    def plugin_get_all(self, context):
        """Return DB entries for all plugins."""
        return self.db.plugin_get_all(context)

    def plugin_update(self, context, name, values):
        """Return updated DB entry for plugin."""
        return self.db.plugin_update(context, name, values)

    def plugin_remove(self, context, name):
        """Remove DB entry for plugin."""
        return self.db.plugin_remove(context, name)



# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.api import acl
from sahara.service.api.v2 import job_types as api
from sahara.service import validation as v
import sahara.utils.api as u


rest = u.RestV2('job-types', __name__)


@rest.get('/job-types')
@acl.enforce("data-processing:job-type:list")
@v.validate_request_params(['type', 'plugin_name', 'plugin_version'])
def job_types_get():
    # We want to use flat=False with to_dict() so that
    # the value of each arg is given as a list. This supports
    # filters of the form ?type=Pig&type=Java, etc.
    request_args = u.get_request_args().to_dict(flat=False)
    if 'plugin_version' in request_args:
        request_args['hadoop_version'] = request_args['plugin_version']
        del request_args['plugin_version']
    return u.render(job_types=api.get_job_types(**request_args))



# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara import conductor as c
from sahara.utils.openstack import base as b
from sahara.utils.openstack import images as sahara_images


conductor = c.API


# Image Registry


def get_images(name, tags):
    return b.execute_with_retries(
        sahara_images.image_manager().list_registered, name, tags)


def get_image(**kwargs):
    if len(kwargs) == 1 and 'id' in kwargs:
        return b.execute_with_retries(
            sahara_images.image_manager().get, kwargs['id'])
    else:
        return b.execute_with_retries(
            sahara_images.image_manager().find, **kwargs)


def get_registered_image(id):
    return b.execute_with_retries(
        sahara_images.image_manager().get_registered_image, id)


def register_image(image_id, username, description=None):
    manager = sahara_images.image_manager()
    b.execute_with_retries(
        manager.set_image_info, image_id, username, description)
    return b.execute_with_retries(manager.get, image_id)


def unregister_image(image_id):
    manager = sahara_images.image_manager()
    b.execute_with_retries(manager.unset_image_info, image_id)
    return b.execute_with_retries(manager.get, image_id)


def get_image_tags(image_id):
    return b.execute_with_retries(
        sahara_images.image_manager().get, image_id).tags


def set_image_tags(image_id, tags):
    manager = sahara_images.image_manager()
    image_obj = b.execute_with_retries(manager.get, image_id)
    org_tags = frozenset(image_obj.tags)
    new_tags = frozenset(tags)

    to_add = list(new_tags - org_tags)
    to_remove = list(org_tags - new_tags)

    if to_add:
        b.execute_with_retries(manager.tag, image_id, to_add)

    if to_remove:
        b.execute_with_retries(manager.untag, image_id, to_remove)

    return b.execute_with_retries(manager.get, image_id)


def remove_image_tags(image_id):
    manager = sahara_images.image_manager()
    image_obj = b.execute_with_retries(manager.get, image_id)
    tags = image_obj.tags
    b.execute_with_retries(manager.untag, image_id, tags)
    return b.execute_with_retries(manager.get, image_id)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from sahara import main
from sahara.tests.unit import base
from sahara.utils import rpc as messaging


class TestMessagingSetup(base.SaharaTestCase):
    def setUp(self):
        super(TestMessagingSetup, self).setUp()
        self.patchers = []
        notifier_init_patch = mock.patch('oslo_messaging.Notifier')
        self.notifier_init = notifier_init_patch.start()
        self.patchers.append(notifier_init_patch)

        get_notif_transp_patch = mock.patch(
            'oslo_messaging.get_notification_transport')
        self.get_notify_transport = get_notif_transp_patch.start()
        self.patchers.append(get_notif_transp_patch)

        get_transport_patch = mock.patch('oslo_messaging.get_rpc_transport')
        self.get_transport = get_transport_patch.start()
        self.patchers.append(get_transport_patch)

        set_def_patch = mock.patch('oslo_messaging.set_transport_defaults')
        self.set_transport_def = set_def_patch.start()
        self.patchers.append(set_def_patch)

    def tearDown(self):
        messaging.NOTIFICATION_TRANSPORT = None
        messaging.MESSAGING_TRANSPORT = None
        messaging.NOTIFIER = None
        for patch in reversed(self.patchers):
            patch.stop()
        super(TestMessagingSetup, self).tearDown()

    def test_set_defaults(self):
        messaging.setup('distributed')

        self.assertIsNotNone(messaging.MESSAGING_TRANSPORT)
        self.assertIsNotNone(messaging.NOTIFICATION_TRANSPORT)
        self.assertIsNotNone(messaging.NOTIFIER)

        expected = [
            mock.call('sahara')
        ]
        self.assertEqual(expected, self.set_transport_def.call_args_list)
        self.assertEqual(
            [mock.call(main.CONF)],
            self.get_transport.call_args_list)
        self.assertEqual(
            [mock.call(main.CONF)],
            self.get_notify_transport.call_args_list)
        self.assertEqual(1, self.notifier_init.call_count)

    def test_fallback(self):
        self.get_notify_transport.side_effect = ValueError()
        messaging.setup('distributed')

        self.assertIsNotNone(messaging.MESSAGING_TRANSPORT)
        self.assertIsNotNone(messaging.NOTIFICATION_TRANSPORT)
        self.assertEqual(
            messaging.MESSAGING_TRANSPORT, messaging.NOTIFICATION_TRANSPORT)
        self.assertIsNotNone(messaging.NOTIFIER)

        expected = [
            mock.call('sahara')
        ]
        self.assertEqual(expected, self.set_transport_def.call_args_list)
        self.assertEqual(
            [mock.call(main.CONF)],
            self.get_transport.call_args_list)
        self.assertEqual(
            [mock.call(main.CONF)],
            self.get_notify_transport.call_args_list)
        self.assertEqual(1, self.notifier_init.call_count)

    def test_only_notifications(self):
        messaging.setup('all-in-one')
        self.assertEqual(0, self.get_transport.call_count)
        self.assertEqual(1, self.get_notify_transport.call_count)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from sahara.tests.unit import base
from sahara.utils import general


class UtilsGeneralTest(base.SaharaWithDbTestCase):
    def setUp(self):
        super(UtilsGeneralTest, self).setUp()

    def test_find_dict(self):
        iterable = [
            {
                "a": 1
            },
            {
                "a": 1,
                "b": 2,
                "c": 3
            },
            {
                "a": 2
            },
            {
                "c": 3
            }
        ]

        self.assertEqual({"a": 1, "b": 2, "c": 3},
                         general.find_dict(iterable, a=1, b=2))
        self.assertIsNone(general.find_dict(iterable, z=4))

    def test_find(self):
        lst = [mock.Mock(a=5), mock.Mock(b=5), mock.Mock(a=7, b=7)]
        self.assertEqual(lst[0], general.find(lst, a=5))
        self.assertEqual(lst[1], general.find(lst, b=5))
        self.assertIsNone(general.find(lst, a=8))
        self.assertEqual(lst[2], general.find(lst, a=7))
        self.assertEqual(lst[2], general.find(lst, a=7, b=7))

    def test_generate_instance_name(self):
        inst_name = "cluster-worker-001"
        self.assertEqual(
            inst_name, general.generate_instance_name("cluster", "worker", 1))
        self.assertEqual(
            inst_name, general.generate_instance_name("CLUSTER", "WORKER", 1))

    def test_get_by_id(self):
        lst = [mock.Mock(id=5), mock.Mock(id=7)]
        self.assertIsNone(general.get_by_id(lst, 9))
        self.assertEqual(lst[0], general.get_by_id(lst, 5))
        self.assertEqual(lst[1], general.get_by_id(lst, 7))

    def test_natural_sort_key(self):
        str_test = "ABC123efg345DD"
        str_list = ['abc', 123, 'efg', 345, 'dd']
        str_sort = general.natural_sort_key(str_test)
        self.assertEqual(len(str_list), len(str_sort))

        self.assertEqual(str_list, str_sort)
